/* $Id: BSpline.h,v 1.3 1992/12/11 08:34:21 jojic Exp $ */
/*****************************************************************************
MODULE:  Geometric
FILE:    BSpline.h
AUTHOR:  Gilles Benati
DATE:    September 20th, 1991

DESCRIPTION:

	This is one of the basic cubic curves. It draws a Tensed B-spline
  i.e: a generalized B-spline with tension. For a discussion about tension see
  GoCubic.h header.
  In the case of tensed B-spline, the tension normally varies from 0 to 1.
  A Tensed B-spline with tension 1 is the normal B-spline. A B-spline is an
  approximating spline i.e.: the curve approximates the control polygon but
  does not pass through them, but it yields a smooth appearance.
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
    
#ifndef _GoBSpline_h
#define _GoBSpline_h

#include <Go/Cubic.h>
#include <Go/mBSpline.h>

class GoBSpline : public GoCubic {
    friend class GoBSplineMetaclass;

  public:

    GoBSpline(int nb = 0 , const GPoint* points = NULL);

    static GoBSplineMetaclass* metaclass();
    static GoBSpline* create(int nb, const GPoint *points = NULL);
    static GoBSpline* create() {
	return (GoBSpline*) Go::Create(GoBSpline::metaclass());
    }

    static int	 save(const GoBSpline*, GoStream*);
    static int	 restore(GoBSpline**, GoStream*);   // creates + restores;
    static int	 readParams(GoBSpline* bspline, GoStream* gostream) {
	return GoCubic::readParams(bspline, gostream); // restores only;
    }

  protected:

//  virtual void m_dup(const Go* src, Go*, Go*, GBool, GBool);
//  virtual GBool m_contains(const GPoint&, GNumber maxDist = 0) const;
//  virtual void m_add_clip(GRegion&) const;
//  virtual void m_opaque_clip(GRegion&) const;
//  virtual void m_draw(const GRegion&) const;
//  virtual void m_wired_draw(const GCoordSys&, const GPen&, GQuality) const;
    
    virtual void initCoeffMatrix();
    
  private:
    
    virtual GoMetaclass* m_metaclass() const;
    virtual void         m_check() const;
};

#endif /* _GoBSpline_h */

