/* $Id: BusLink.h,v 1.6 1992/12/11 08:34:26 jojic Exp $ */
/*****************************************************************************
MODULE:  BULL Graphic Objects
FILE:    BusLink.h
AUTHOR:  Aline Jacomet
DATE:    January 23th 1991

DESCRIPTION:
         Definition of the bus link object class.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _GoBusLink_h
#define _GoBusLink_h

#include <Go/Go.h>
#include <Go/Link.h>
#include <Go/mBusLink.h>
#include <Go/Node.h>
#include <Go/Tools/Pen.h>



class GoBusLink : public GoLink {
    friend class GoBusLinkMetaclass;

  public:
    /*
     * get ...;
     */
    virtual GPen& pen() const {
	return ((GoBusLink*)this)->_pen;
    };
    inline short orientation() const {
	return _orientation;
    }

    /*
     * set ...;
     */
    GBool orientation(short);
    
    /*
     * others ...;
     */
    static GoBusLinkMetaclass* metaclass();
    static GoBusLink*          create();
    static GoBusLink*          create(short);

    /*
     * save, restore;
     */
    static int save(const GoBusLink*, GoStream*);
    static int restore(GoBusLink**, GoStream*);
    static int readParams(GoBusLink*, GoStream*);

  protected:
    virtual void m_dup(const Go*, Go*, Go*, GBool, GBool);

    GoBusLink();
    virtual ~GoBusLink();

  private:
    GPen    _pen;
    GShPen* _old_sh_pen;
    short _orientation;

    virtual GoMetaclass* m_metaclass() const;
    virtual void         m_draw(const GRegion&) const;
    virtual void         m_add_clip(GRegion&) const;
    virtual GBool        m_contains(const GPoint&, GNumber maxDist=0) const;

    void modify_pen(GPen*, ulong);
    friend void buslink_modify_pen(void*, GPen*, ulong);
    void calculate_points(GPoint*) const;

    const GShPen* sh_pen() const;
    void init_sh_pen() {
	if (_old_sh_pen)
	    _old_sh_pen->destroy();
	_old_sh_pen = _pen.shPen()->dup();
    }
};

INLINE GoBusLink* GoBusLink::create() {
    return (GoBusLink*) Go::Create(GoBusLink::metaclass());
}

INLINE GoBusLink* GoBusLink::create(short orientation) {
    GoBusLink* link =  (GoBusLink*) Go::Create(GoBusLink::metaclass());
    link->_orientation = orientation;
    return link;
}


#endif /* _GoBusLink_h */
