/* $Id: CSGrabber.h,v 1.2 1992/12/16 16:25:35 jojic Exp $ */
 /*****************************************************************************
MODULE:  Go Kernel (User Interface)
FILE:    CSGrabber.h
AUTHOR:  Oliver Jojic
DATE:    July 92

DESCRIPTION:

COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/
#ifndef _CSGrabber_h
#define _CSGrabber_h
    
#include <Go/Grabber.h>
#include <Go/Tools/GCdef.h> 
#include <Go/Tools/CoordSys.h>
#include <Go/Tools/Queue.h>
#include <Go/Tools/Pen.h>

/*
  *-------------------------------------------------------
  * Programmation of a CSGrabber
  *-------------------------------------------------------
  */

class GoCSGrabberPgm : public GGrabberPgm
{
  public:

    GBool callCSPreconds () const { return call_cs_preconds; }
    void callCSPreconds (GBool b) { call_cs_preconds = b; }

    int maxDrawCount () const { return max_count; }
    int maxDrawDepth () const { return max_depth; }
    void maxDrawCount (int n) { max_count = n; }
    void maxDrawDepth (int n) { max_depth = n; }
    GQuality drawingQuality() const { return _drawing_quality;}
    void     drawingQuality(GQuality quality) { _drawing_quality = quality; }
    GPen&    drawingPen() const;
    
    void collectGosToBeDrawn (GQueue&, Go*) const;

  protected:
    GoCSGrabberPgm (const char*);
    
  private:

    /* private data */
    GBool call_cs_preconds;
    int max_count;
    int max_depth;
    GQuality _drawing_quality;
    GPen     _drawing_pen;
};


extern void GCollectSomeGosFromThisTree (GQueue&, Go*, int max_depth, int max_count);


/*
  *-------------------------------------------------------
  * Execution of a GoCSGrabber
  *-------------------------------------------------------
  */

class GoCSGrabber : public GGrabber 
{
  public:

    /* superclasses virtual methods */
    virtual void doit (); //superclass first
    virtual void cancel (); //superclass first
    virtual void draw ();
    virtual void clear ();

  protected:
    GoCSGrabber (GoCSGrabberPgm*, const GEvent*, Go*);
    const GoCSGrabberPgm* Pgm () const { return (GoCSGrabberPgm*) pgm(); }

    virtual void Initialize (); //superclass last

    /* protected data */
    GCoordSys _cs;
    GQueue drawn_gos;
    GBool change_subtree_cs;
    GPen  _drawing_pen;        // copied from pgm;
    GQuality _drawing_quality; // copied from pgm;
    
    friend GoCSGrabberPgm;
};

inline GPen& GoCSGrabberPgm::drawingPen() const
{
    return ((GoCSGrabberPgm*)this)->_drawing_pen;
}
#endif /* _CSGrabber_h */
