/* $Id: Composite.h,v 1.3 1992/12/11 08:34:30 jojic Exp $ */
/*****************************************************************************
MODULE:  Basic Graphic Objects
FILE:    Composite.h
AUTHOR:  Oliver Jojic
DATE:    April 91

DESCRIPTION:
  The GoComposite class is an instanciable go that:
  	- make no drawings,
	- stores an absolute coordinate system,
	- is used to store children.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _GoComposite_
#define _GoComposite_

#include <Go/GoAbsDS.h>
#include <Go/mComposite.h>


/***************************************************
*
*		the GoComposite class
*
****************************************************/


class GoComposite : public GoAbsDS
{
  public:
    static GoCompositeMetaclass*  metaclass ();
    static GoComposite*	 	  create ();

  protected:
    virtual void	 m_dup (const Go*, Go*, Go*, GBool, GBool);

    /*
     * save, restore;
     */
    static int save(const GoComposite*, GoStream*);
    static int restore(GoComposite**, GoStream*);
    static int readParams(GoComposite*, GoStream*);

  private:
    virtual GoMetaclass* m_metaclass () const;
    virtual GBool	 m_contains (const GPoint&, GNumber) const;

    virtual GRegion*	 d_get_clip () const;

    /*** data ***/
    GRegion _clip;
};


/*************************************************
*
*		inline methods
*
*************************************************/

INLINE GoComposite* GoComposite::create ()
{
    return (GoComposite*)Go::Create((GoMetaclass*)GoComposite::metaclass());
}

#endif /* _GoComposite_ */
