/* $Id: Curve.h,v 1.3 1992/12/11 08:34:33 jojic Exp $ */
/*****************************************************************************
MODULE: Geometric
FILE:   Curve.h
AUTHOR: Gilles Benati
DATE:   September 20th, 1991

DESCRIPTION :

        This is a base class for all the curve sub classes. This class provides
    the common functions for the sub classes like drawing the curve during
    fast Redraw, drawing of the curve during interactive manipulation of
    control points, checking whether the curve contains a point etc. In order
    to utilise these functions the sub class has to implement the virtual
    function 'getCurvePoints' which returns the points on the curve with a
    specified precision.
      Point* getCurvePoints (int& np)

        A positive value of np on entry specifies the desired number of points
    on the  curve. On exit np contains actual number of points returned. The
    number of points returned by the sub class should be as close as possible
    to the desired number, but at the same time should ensure that it is
    sufficient to draw the curve as a 'curve'. if np is negative, the subclass
    can choose it's most optimum number of points required to draw the curve.
 
COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/
#ifndef _GoCurve_h
#define _GoCurve_h

#include <Go/Polypoint.h>
#include <Go/mCurve.h>

class GoCurve : public GoPolypoint {
    friend class GoCurveMetaclass;

  public:
    
    static GoCurveMetaclass* metaclass();
    
    virtual GBool delPoint(int index);

    static int	 save(const GoCurve* curve, GoStream* gostream) {
	return GoPolypoint::save(curve, gostream);
    }
//  static int	 restore(GoCurve**, GoStream*);   // creates + restores;
    static int	 readParams(GoCurve* curve, GoStream* gostream) {
	return GoPolypoint::readParams(curve, gostream); // restores only;
    }

  protected:

    virtual GBool m_contains(const GPoint&, GNumber maxDist = 0) const;
    virtual void m_draw(const GRegion&) const;
    virtual void m_wired_draw(const GCoordSys&, const GPen&, GQuality) const;
    virtual void m_add_clip(GRegion&) const;
//  virtual void m_add_opaque_clip(GRegion&) const;
//  virtual void m_dup(const Go*, Go*, Go*, GBool, GBool);

    virtual GPoint* getCurvePoints(int& np, int nb_ctlr,
				   const GPoint* ctrl) const = 0;

    GoCurve(int npoint = 0, const GPoint* = NULL);

//  GPoint getNewPosition(int index, GBool& ok);
//  GPoint getNewPoint(int index, GBool& ok);

  private:
    
    virtual GoMetaclass* m_metaclass() const;
    virtual void         m_check() const;

//  void movePointProc(GPoint*, GPoint*, GPoint*, GoEvent*);
//  void addPointProc(GPoint*, GPoint*, GPoint*, GoEvent*);
//  GPoint getAddMovePoint(int index, GBool& ok, Proc proc);
};

#endif /* _GoCurve_h */
