/* $Id: EdArc.h,v 1.2 1992/12/30 15:55:38 smadja Exp $ */
/*****************************************************************************
MODULE:  BGo  (User Interface)
FILE:    EdArc.h
AUTHOR:  Philippe Smadja
DATE:    July 92

DESCRIPTION:

  GEditArc is a subclass of GEditAsArc
  
  It uses the superclass to compute interaction. It uses a arc to
  initialize value at the start time and reset arc (doit method).
	 
COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/
#ifndef _GEdArc_H
#define _GEdArc_H

#include <Go/EdGeometric.h>

/*
 * values for arc manipulation cf mask parameter
 */

#define G_DO_RADX     1<<0
#define G_DO_RADY     1<<1
#define G_DO_RADII    1<<2
#define G_DO_END      1<<3
#define G_DO_BEGIN    1<<4

// convenient values;

#define G_DO_NOTHING  0
#define G_DO_IT       1<<5
#define G_DO_ANYTHING (G_DO_RADX|G_DO_RADY|G_DO_RADII|G_DO_END|G_DO_BEGIN)

// Note that G_DO_RADX | G_DO_RADY is a valid choice

/*
  *-------------------------------------------------------
  * The GEditArcPgm class
  * The class is not instanciable. It is just the super class
  * of GEditArcBeginPgm,  GEditArcEndPgm,  GEditArcRadiiPgm, 
  *-------------------------------------------------------
  */

    
class GEditArcBeginPgm : public GEditGeometricPgm
{
  public:
    static GEditArcBeginPgm* create (); // my name is "GoArc.editBegin"

    // superclasses virtual method;
    virtual GInteractor* start (const GEvent*, Go*);

  protected:
    GEditArcBeginPgm (const char* name) : GEditGeometricPgm(name) {};
};

class GEditArcEndPgm : public GEditGeometricPgm
{
  public:
    static GEditArcEndPgm* create (); // my name is "GoArc.editEnd";

    // superclasses virtual method;
    virtual GInteractor* start (const GEvent*, Go*);

  protected:
    GEditArcEndPgm (const char* name) : GEditGeometricPgm(name) {};
};

class GMoveArcPointPgm : public GEditGeometricPgm
{
  public:
    static GMoveArcPointPgm* create (); // my name is "GoArc.editEnd";

    // superclasses virtual method;
    virtual GInteractor* start (const GEvent*, Go*);

  protected:
    GMoveArcPointPgm (const char* name) : GEditGeometricPgm(name) {};
};

class GEditArcRadiiPgm : public GEditGeometricPgm
{
  public:
    static GEditArcRadiiPgm* create (); // my name is "GoArc.editRadii";

    // superclasses virtual method;
    virtual GInteractor* start (const GEvent*, Go*);

  protected:
    GEditArcRadiiPgm (const char* name) : GEditGeometricPgm(name) {};
};


class GEditArcRadxyPgm : public GEditGeometricPgm
{
  public:
    static GEditArcRadxyPgm* create (); // my name is "GoArc.editRadxy";

    // superclasses virtual method;
    virtual GInteractor* start (const GEvent*, Go*);

  protected:
    GEditArcRadxyPgm (const char* name) : GEditGeometricPgm(name) {};
};


/*
 *-------------------------------------------------------
 * The GEditArc class
 *-------------------------------------------------------
 */

class GoArc;

class GEditArc : public GEditGeometric
{
  public:

    GEditArc (GEditGeometricPgm*, const GEvent*, GoArc*, int mask);
    virtual ~GEditArc ();

    virtual void  draw ();
    virtual void  doit ();
    virtual void  compute(const GEvent*);

    // read mask;
    int mask() const { return _mask; } ;

    // set mask;
    void mask(int);

  protected:
    virtual void Initialize();
    void MyCompute();

  private:
    int _mask;          // cf gravity definition
    GNumber _radiusX;
    GNumber _radiusY;
    GNumber _angle1;
    GNumber _angle2;
    GBool   _goWasMapped;
};

#endif /* _GEdArc_H */


