/* $Id: EdRegPolygon.h,v 1.1 1992/12/10 15:37:28 jojic Exp $ */
/*****************************************************************************
MODULE:  BGo  (User Interface)
FILE:    EdRegPoly.h
AUTHOR:  Philippe Smadja
DATE:    July 92

DESCRIPTION:

  GEditRectangle is a subclass of GGrabber
  
  GRectabgleGrabber is used for the programming of user interactions ala
  ``elastic rectangle''.
  Such interactions follow the same scheme: The Grabber one.
	- drawings,
	- wait for a motion event,
	- clear the previous drawings,
	- re-computation of data values (using the new mouse position),

	This loop stops on some specific user events, which may generate a succes
	or a cancellation of the interaction.

	Every action from the list above, correspond to a virtual method of
	the GGrabber class. The virtual method ``handle(event)''
	executes previous actions.

  The GGrabber::handle method is fine for GGrabber so we have only to implement
  the following methods:
         draw();
	 compute();
	 doIt();
	 isDoItEvent();
	 isCancelEvent();
	 
COPYRIGHT:
  Copyright (c) 1990, 1991, 1992 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software

  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.

*****************************************************************************/
#ifndef _GEdRegPolygon_H
#define _GEdRegPolygon_H

#include <Go/EdGeometric.h>
#include <Go/Tools/CoordSys.h>

/*
  *-------------------------------------------------------
  * The GEditRegPolygonPgm class
  *-------------------------------------------------------
  */

class GEditRegPolygonPgm : public GEditGeometricPgm
{
  public:
    static GEditRegPolygonPgm* create (); 

    // superclasses virtual method;
    virtual GInteractor* start (const GEvent*, Go*);

  protected:
    GEditRegPolygonPgm (const char*);
};


class GoRegPolygon;
class GEvent;

/*
 *-------------------------------------------------------
 * The GEditRegPolygon class
 *-------------------------------------------------------
 */

class GEditRegPolygon : public GEditGeometric
{
  public:

    GEditRegPolygon (GEditRegPolygonPgm*, const GEvent*, GoRegPolygon*);
    virtual ~GEditRegPolygon ();

    virtual void  draw ();
    virtual void  compute (const GEvent*); //superclass first;
    virtual void  doit ();

  protected:

    virtual void  Initialize ();
    GQuality      DrawingQuality();
    void          MyCompute();
    
  private:
    
    GCoordSys _deform_cs;
    int       _nb;
};

    
#endif /* _GEdRegPoly_H */


