/* $Id: Ellipse.h,v 1.3 1992/12/11 08:34:34 jojic Exp $ */
/*****************************************************************************
MODULE:  Graphic Objects
FILE:    Ellipse.h
AUTHOR:  Gilles Benati
DATE:    Feb 1991

DESCRIPTION:

        This is one of the basic graphic objects, subclass of GoArc.
    _amplitude and _start angles are forever set respecitvely to 2*PI and 0.
    _pie and _closed attributes are forever set to FALSE;

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/
#ifndef _GoEllipse_h
#define _GoEllipse_h

#include <Go/Arc.h>
#include <Go/mEllipse.h>

class GoEllipse : public GoArc {
    friend class GoEllipseMetaclass;

  public:

    static GoEllipseMetaclass* metaclass();
    static GoEllipse* create(GNumber radiusX, GNumber radiusY = 1.);
    static GoEllipse* create() {
	return (GoEllipse*) Go::Create(GoEllipse::metaclass());
    }
    
    GoEllipse(GNumber radX=1., GNumber radY=1.) : GoArc(radX, radY, 2*PI, 0.) { }
    virtual GBool reset(GNumber radX, GNumber radY, GNumber = 0., GNumber = 0.);

    static int	 save(const GoEllipse*, GoStream*);
    static int	 restore(GoEllipse**, GoStream*);   // creates + restores;
    static int	 readParams(GoEllipse*, GoStream*); // restores only;


  protected:
    
    virtual GBool m_contains(const GPoint&, GNumber maxDist = 0) const;
    virtual void m_add_clip(GRegion&) const;
    virtual void m_add_opaque_clip(GRegion&) const;
    virtual void m_draw(const GRegion&) const;
    virtual void m_wired_draw(const GCoordSys&, const GPen&, GQuality) const;
//  virtual void m_dup(const Go* src, Go*, Go*, GBool, GBool);

  private:
    
    virtual GoMetaclass* m_metaclass() const;
    virtual void         m_check() const;

    virtual GBool start(GNumber);     //  {return GFALSE};
    virtual GBool amplitude(GNumber); //  {return GFALSE};
    virtual GBool pie(GBool);         //  {return GFALSE};
    virtual GBool closed(GBool);      //  {return GFALSE};
};

#endif /* _GoEllipse_h */
