/* $Id: Form.h,v 1.5 1992/11/12 15:36:46 andre Exp $ */
/*****************************************************************************
MODULE:  GoText
FILE:    Form.h
AUTHOR:  Michel Andre
DATE:    August 91
 

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/

#ifndef         _Form_h
#define         _Form_h

#include <Go/Tools/Region.h>
#include <Go/TextDefs.h>
#include <Go/machdef.h>


class Go;
class GoStream;

class GFormatterCtxt{

friend class GFormatter;
friend class GSimpleFormatter;
friend class GFrFormatter;
friend class GMCFormatter;
friend class GMBFormatter;
friend class GCircleFormatter;

protected:
			GFormatterCtxt();
    GFormatter*		_formatter;
};

class GFormatter{
    friend class        GoContainer;
    friend class        GoTextSegment;
    friend class        GoTextLine;
    friend class        GoText;
    friend class	GFormatterCtxt;
    friend class	GSimpleFormatterCtxt;
    friend class	GFrFormatterCtxt;
    friend class	GMCFormatterCtxt;
    friend class	GMBFormatterCtxt;
    friend class	GCircleFormatterCtxt;

public:
			GFormatter();
    virtual		~GFormatter();
    GFormatterCtxt*	ctxt() const {return ((GFormatter*)this)->_ctxt;}
    virtual const char*	getClassname() const;
    static const char*	classname();
    static int		save (const GFormatter*, GoStream*)
			{return 1;}
    static int		readParams (GFormatter*, GoStream*) {return 1;}
    virtual GBool	isFrameFormatter() const {return GFALSE;}
    void                enableReframe() {_enableReframe = GTRUE;}
    void                disableReframe() {_enableReframe = GFALSE;}
    virtual void	reframe(){}
    virtual void	justification (GJustification);

protected:
    GoText*		_text;
    GFormatterCtxt*	_ctxt;
    GBool               _enableReframe;

    virtual GFormatter*	dup(){return NULL;}
    virtual void	clip (GRegion&) const;
    virtual void	draw (const GRegion&) const;
    virtual void	setString();
    virtual void	insertChar (char, GPos){}
    virtual void	insertString (const char*, GPos, GPos){}
    virtual void	deleteChar (char, GPos){}
    virtual void	deleteString (const char*, GPos, GPos){}
    virtual void	replaceChar (char, GPos){}
    virtual void	replaceString (GPos, GPos, char*, GPos){}
    virtual void	assembly (GoTextSegment*, GoTextSegment*){};
    virtual void	organize (Go* = NULL, Go* = NULL);
    void		refreshLine (GoTextLine*) const;
    virtual GNumber	indentation (GoTextLine*) const {return 0;}
    virtual GNumber     textWidth (GoTextLine*) const;
    virtual GPoint	lineOrigin (GoTextLine*) const;
    virtual GoTextLine*	xyToLine (const GPoint&) const;
    virtual GPos	xyToPos (GPoint&) const;
    virtual GPoint	posToXy (GPos) const;
};

#endif                  /* _Form_h */
