/* $Id: Geometric.h,v 1.3 1992/12/11 08:34:39 jojic Exp $ */
/*****************************************************************************
MODULE:  Graphic Objects
FILE:    Geometric.h
AUTHOR:  Gilles Benati
DATE:    Feb 1991

DESCRIPTION:

      This class is a basic class for every geometric graphic object
  (line, polyline, rectangle, circle, arc, curves ...).
    
COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/
#ifndef _GoGeometric_h
#define _GoGeometric_h

#include <Go/GoStdDS.h>
#include <Go/Tools/Pen.h>
#include <Go/Tools/Brush.h>
#include <Go/mGeometric.h>

class GoGeometric : public GoStdDS {
    friend class GoGeometricMetaclass;
    
  public:

    static GoGeometricMetaclass* metaclass();

    // get values...;
    // some silly compilers complain if "return _pen" or "return _brush";
    GPen&   pen()        const { return ((GoGeometric*) this)->_pen; }
    GBrush& brush()      const { return ((GoGeometric*) this)->_brush; }
    GBool   outline()    const { return _outline; }
    GBool   background() const { return _background; }

    // set values...;
    virtual GBool outline(GBool);
    virtual GBool background(GBool);
    
    // pre/post conditions;
    void outlinePrecond(GBool func(void*, const GoGeometric*, GBool*),
			void* closure, GAddOrRemove = GADD);
    void outlinePostcond(void func(void*, GoGeometric*),
			 void* closure, GAddOrRemove = GADD);
    void backgroundPrecond(GBool func(void*, const GoGeometric*, GBool*),
			   void* closure, GAddOrRemove = GADD);
    void backgroundPostcond(void func(void*, GoGeometric*),
			    void* closure, GAddOrRemove = GADD);

    static int	 save(const GoGeometric*, GoStream*);
//  static int	 restore(GoGeometric**, GoStream*);   // creates + restores;
    static int	 readParams(GoGeometric*, GoStream*); // restores only;

  protected:


    virtual void m_dup(const Go* src, Go*, Go*, GBool, GBool);
    
    GoGeometric();
    ~GoGeometric();

    const GShPen* sh_pen() const { return _old_sh_pen; }
    void  init_sh_pen() {
	if (_old_sh_pen)
	    _old_sh_pen->destroy();
	_old_sh_pen = _pen.shPen()->dup();
    }
    
  private:

    // pre/post conditions flags;
    GBool _has_outline_precond		:1;
    GBool _has_outline_postcond		:1;
    GBool _has_background_precond	:1;
    GBool _has_background_postcond	:1;
    
    GBool _outline			:1;        // has an outline or not;
    GBool _background			:1;        // has a background or not;
    
    GShPen* _old_sh_pen;
    GPen    _pen;                  // used to draw outline;
    GBrush  _brush;                // used to draw background;

    // private postconditions on _brush and _pen modifications;
    static void _pen_changed(GoGeometric*, const GPen*, ulong);
    static void _brush_changed(GoGeometric*, const GBrush*, ulong);
    
    // call pre/post conditions;
    GBool _call_outline_precond(GBool*) const;
    void _call_outline_postcond();
    GBool _call_background_precond(GBool*) const;
    void _call_background_postcond();

    // call metaclasses pre/post conditions;
    GBool _call_class_outline_precond(GBool*) const;
    void _call_class_outline_postcond();
    GBool _call_class_background_precond(GBool*) const;
    void _call_class_background_postcond();
    
    GBool _set_outline(GBool);
    GBool _set_background(GBool);
    
    virtual GoMetaclass* m_metaclass() const;
    virtual void         m_check() const;
};

#endif /* _GoGeometric_h */
