/* $Id: GoTreeDS.h,v 1.3 1992/12/11 08:34:51 jojic Exp $ */
/*****************************************************************************
MODULE:  Go Kernel
FILE:    GoTreeDS.h
AUTHOR:  Oliver Jojic
DATE:    April 91

DESCRIPTION:
  The Go class itself do not have any data storage. This allow implementation
  of "light weight gos".
  The GoTreeDS class (DS means data storage) is a light weight
  implementation of data storage.
  The GoTreeDS class cannot be instanciated.
  The GoTreeDS class is a good superclass for light weight gos that may
  have children.

  The GoTreeDS class:
  	- draws nothing,
	- has no assoc,
	- has the coordinate system of its parent,
	- do not store the clip of itself and its children.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _GoTreeDS_
#define _GoTreeDS_

#include <Go/Go.h>


/***************************************************
*
*		the GoTreeDS class
*
****************************************************/


class GoTreeDS : public Go
{
  public:
    static GoMetaclass*  metaclass ();
    static int		 save (const GoTreeDS*, GoStream*);
    static int		 readParams (GoTreeDS*, GoStream*);

  protected:
    			 GoTreeDS ();
    virtual    		~GoTreeDS ();

    virtual void	 m_dup (const Go*, Go*, Go*, GBool, GBool);
  private:
    virtual GoMetaclass* m_metaclass () const;

    virtual GAssoc*	 d_get_assoc () const;
    virtual Go*		 d_get_parent () const;
    virtual Go*		 d_get_previous () const;
    virtual Go*		 d_get_next () const;
    virtual Go*		 d_get_first_child () const;
    virtual GBool	 d_store_children_array () const;
    virtual void	 d_set_previous (const Go*);
    virtual void	 d_set_next (const Go*);
    virtual void	 d_remove_child (const Go*, int);
    virtual void	 d_add_child (const Go*, int);
    virtual void	 d_set_tree (const Go*, const Go*, const Go*);
    virtual GBool	 d_store_abscs () const;
    virtual GBool	 d_store_relcs () const;
    virtual const GCoordSys& d_get_relcs (const GCoordSys*) const;
    virtual void 	 d_set_coordsys (const GCoordSys&, const GCoordSys&,
					 const GCoordSys&);

    /*** data ***/
    Go* _parent;
    Go* _next;
    Go* _child;
};


/**********************************************
*
*		inline methods
*
***********************************************/

inline GoTreeDS::GoTreeDS ()
{
    _parent = NULL;
    _next = NULL;
    _child = NULL;
}

#endif /* _GoTreeDS_ */
