/* $Id: GraphNode.h,v 1.5 1992/12/11 08:34:55 jojic Exp $ */
/*****************************************************************************
MODULE:  BULL Graphic Objects
FILE:    GraphNod.h
AUTHOR:  Aline Jacomet
DATE:    February 8th 1991

DESCRIPTION:
         Definition of the graph node object class. 

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _GoGraphNode_h
#define _GoGraphNode_h

#include <Go/Go.h>
#include <Go/Node.h>
#include <Go/mGraphNode.h>
#include <Go/Link.h>
#include <Go/Tools/Array.h>


/*
 * GoGraphNode definition
 */
class GoGraphNode : public GoNode {
    friend class GoGraph;
    friend class GoLink;
    friend class GoGraphNodeMetaclass;
    friend class GoGraphStruct;
    
  public:
    /*
     * get ...;
     */
    GoLink** nextLink() const;
    GoLink** previousLink() const;
    GoLink** allLink() const;
    GoLink*  link(const GoGraphNode*) const;
    GoGraphNode** nextNode() const;
    GoGraphNode** previousNode() const;
    GoGraphNode** allNode() const;
    inline ushort nbPreviousLink() const {
	return prev_link.nelems();
    }
    inline ushort nbNextLink() const {
	return next_link.nelems();
    }
    
    /*
     * Pre and Post Conditions ...;
     */
    void attachNodePrecond(GBool (*)(void*, GoGraphNode*, GoGraphNode*, GBool*,
				     GoMetaclass*, GoGraph*),
			   void* closure, GAddOrRemove = GADD);
    void attachNodePostcond(void (*)(void*, GoGraphNode*, GoGraphNode*, GBool*,
				     GoMetaclass*, GoGraph*),
			    void* closure, GAddOrRemove = GADD);
    
   /*
     * others ...;
     */
    static GoGraphNodeMetaclass* metaclass();
    static GoGraphNode*          create();
    
    GBool attachNode(GoGraphNode*, GBool is_previous = GTRUE,
		     GoMetaclass* linkclass = NULL, GoGraph* graph = NULL);

    /*
     * save, restore;
     */
    static int save(const GoGraphNode*, GoStream*);
    static int restore(GoGraphNode**, GoStream*);
    static int readParams(GoGraphNode*, GoStream*);

  protected:
    GoGraphNodeCondFlags _flags;

    virtual void m_dup(const Go*, Go*, Go*, GBool, GBool);
    GoGraphNode();
    virtual ~GoGraphNode();
    
    GBool _call_attach_node_precond(GoGraphNode*, GBool*, GoMetaclass*,
				    GoGraph*) const;
    void _call_attach_node_postcond(GoGraphNode*, GBool*, GoMetaclass*,
				    GoGraph*);
    GBool _call_class_attach_node_precond(GoGraphNode*, GBool*, GoMetaclass*,
					  GoGraph*) const;
    void _call_class_attach_node_postcond(GoGraphNode*, GBool*, GoMetaclass*,
					  GoGraph*);
    
  private:
    GPointerArray prev_link;
    GPointerArray next_link;

    virtual GoMetaclass* m_metaclass() const;
    
    void reparent_on_graph(GoGraph*);
};

INLINE GoGraphNode* GoGraphNode::create() {
    return (GoGraphNode*) Go::Create(GoGraphNode::metaclass());
}

#endif /* _GoGraphNode_h */
