/* $Id: Polyline.h,v 1.3 1992/12/11 08:35:09 jojic Exp $ */
/*****************************************************************************
MODULE: Graphic Objects
FILE:   Polyline.h
AUTHOR: Oliver Jojic, Gilles Benati
DATE:   Feb 1991

DESCRIPTION :

    This is one of the basic graphic objects. The origin of its coord sys is
not related to any of the points in the GoPolypoint array points(), i.e. adding
or (re)moving points will not affect the coord sys.
    
COPYRIGHT :
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/
#ifndef _GoPolyline_h
#define _GoPolyline_h

#include <Go/Polypoint.h>
#include <Go/mPolyline.h>

class GoPolyline : public GoPolypoint {

    friend class GoPolylineMetaclass;
    
  public:

    static GoPolylineMetaclass* metaclass();
    static GoPolyline* create(int nbPoint, const GPoint* pts);
    static GoPolyline* create() {
	return (GoPolyline*) Go::Create(GoPolyline::metaclass());
    }

    // pre/post conditions;
    void roundPrecond(GBool func(void*, const GoPolyline*, GBool*),
		      void* closure, GAddOrRemove = GADD);
    void roundPostcond(void func(void*, GoPolyline*),
		       void* closure, GAddOrRemove = GADD);
    void roundOffsetPrecond(GBool func(void*, const GoPolyline*, GNumber*),
			    void* closure, GAddOrRemove = GADD);
    void roundOffsetPostcond(void func(void*, GoPolyline*),
			     void* closure, GAddOrRemove = GADD);
    
    GoPolyline();
    GoPolyline(int npoint, const GPoint*);

    GBool         round()       const { return _round; }
    GNumber       roundOffset() const { return _round_offset; }
    virtual GBool round(GBool);
    virtual GBool roundOffset(GNumber);
    virtual GBool delPoint(int index);

    static int	 save(const GoPolyline*, GoStream*);
    static int	 restore(GoPolyline**, GoStream*);   // creates + restores;
    static int	 readParams(GoPolyline*, GoStream*); // restores only;

  protected:


    virtual GBool m_contains(const GPoint&, GNumber maxDist = 0) const;
    virtual void m_add_clip(GRegion&) const;
//  virtual void m_add_opaque_clip(GRegion&) const;
    virtual void m_draw(const GRegion&) const;
    virtual void m_wired_draw(const GCoordSys&, const GPen&, GQuality) const;
    virtual void m_dup(const Go* src, Go*, Go*,GBool, GBool);
    
    void DeltaVect(const GPoint&, const GPoint&, const GPoint&,
		   GVector&, GVector&) const;
    
    GNumber _round_offset;
    GBool   _round                  :1;
    
  private:

    // pre/post condition flags;
    GBool _has_round_precond         :1;
    GBool _has_round_postcond        :1;
    GBool _has_round_offset_precond  :1;
    GBool _has_round_offset_postcond :1;
    
    // call pre/post conditions;
    GBool _call_round_precond(GBool*) const;
    GBool _call_round_offset_precond(GNumber*) const;
    void _call_round_postcond();
    void _call_round_offset_postcond();

    // call metaclass pre/post conditions;
    GBool _call_class_round_precond(GBool*) const;
    GBool _call_class_round_offset_precond(GNumber*) const;
    void _call_class_round_postcond();
    void _call_class_round_offset_postcond();

    GBool _set_round(GBool);
    GBool _set_round_offset(GNumber);
    
    virtual int get_round_points(GPoint** roundPoints) const;
    
    virtual GoMetaclass* m_metaclass() const;
    virtual void         m_check() const;
};

#endif /* _GoPolyline_h */


