/* $Id: Polypoint.h,v 1.3 1992/12/11 08:35:10 jojic Exp $ */
/*****************************************************************************
         Copyright (c) BULL S.A 1990

MODULE: Graphic Objects
FILE:   Polypoint.h
AUTHOR: Aboobacker T.A
DATE:   Feb 1991

DESCRIPTION :

    This is the base class for polyline and curve families. It manages an
array of points and provides common functions to manipulate this array :
reset the array, add, move or delete a point...
    The origin of the coord system is not related to any of the point in the
point array, i.e. adding or (re)moving points will not affect the coord sys.

COPYRIGHT :
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/
#ifndef _GoPolypoint_h
#define _GoPolypoint_h

#include <Go/Geometric.h>
#include <Go/mPolypoint.h>

class GoPolypoint : public GoGeometric {
    friend class GoPolypointMetaclass;

  public:

    static GoPolypointMetaclass* metaclass();

    // pre/post conditions;
    void addPointPrecond(GBool func(void*, const GoPolypoint*, int*, GPoint*),
			 void* closure, GAddOrRemove = GADD);
    void addPointPostcond(void func(void*, GoPolypoint*, int),
			  void* closure, GAddOrRemove = GADD);
    void delPointPrecond(GBool func(void*, const GoPolypoint*, int*),
			 void* closure, GAddOrRemove = GADD);
    void delPointPostcond(void func(void*, GoPolypoint*, int),
			  void* closure, GAddOrRemove = GADD);
    void movePointPrecond(GBool func(void*, const GoPolypoint*, int*, GPoint*),
			  void* closure, GAddOrRemove = GADD);
    void movePointPostcond(void func(void*, GoPolypoint*, int),
			   void* closure, GAddOrRemove = GADD);
    void resetPointsPrecond(GBool
			    func(void*, const GoPolypoint*, int, const GPoint*),
			    void* closure, GAddOrRemove = GADD);
    void resetPointsPostcond(void func(void*, GoPolypoint*),
			     void* closure, GAddOrRemove = GADD);
    
    // call preconditions (in public);
    GBool callAddPointPrecond(int*, GPoint*) const;
    GBool callDelPointPrecond(int*) const;
    GBool callMovePointPrecond(int*, GPoint*) const;
    GBool callResetPointsPrecond(int, const GPoint*) const;

    // get values...;
    const GPoint* points()        const { return _points; }
    int nbPoint()                const { return _nbPoint; }
    int posToIndex(const GPoint&) const;
    
    // set values...;
    virtual GBool reset(int nbPoint, const GPoint*);
    
    // miscellaneous;
    virtual GBool addPoint(int index, const GPoint&, GBool isAbsPoint);
    virtual GBool movePoint(int index, const GPoint&, GBool isAbsPoint);
    virtual GBool delPoint(int index);

    static int	 save(const GoPolypoint*, GoStream*);
//  static int	 restore(GoPolypoint**, GoStream*);   // creates + restores;
    static int	 readParams(GoPolypoint*, GoStream*); // restores only;


  protected:

    int _nbPoint;
    GPoint* _points;
    
    virtual GBool m_contains(const GPoint&, GNumber maxDist = 0) const;
    virtual void m_dup(const Go* src, Go*, Go*,GBool, GBool);
//  virtual void m_add_clip(GRegion&) const;
//  virtual void m_add_opaque_clip(GRegion&) const;
//  virtual void m_draw(const GRegion&) const;
//  virtual void m_wired_draw(const GCoordSys&, const GPen&, GQuality) const;

    void new_points(int n, const GPoint*, GBool remove_old_pts = GTRUE);
    
    GoPolypoint(int nbPoint = 0, const GPoint* = NULL);
    ~GoPolypoint();
        
  private:

    // pre/post condition flags;
    GBool _has_add_point_precond     :1;
    GBool _has_add_point_postcond    :1;
    GBool _has_del_point_precond     :1;
    GBool _has_del_point_postcond    :1;
    GBool _has_move_point_precond    :1;
    GBool _has_move_point_postcond   :1;
    GBool _has_reset_points_precond  :1;
    GBool _has_reset_points_postcond :1;
    
    // call pre/post conditions;
    GBool _call_add_point_precond(int*, GPoint*) const;
    GBool _call_del_point_precond(int*) const;
    GBool _call_move_point_precond(int*, GPoint*) const;
    GBool _call_reset_points_precond(int, const GPoint*) const;
    void _call_add_point_postcond(int);
    void _call_del_point_postcond(int);
    void _call_move_point_postcond(int);
    void _call_reset_points_postcond();
    
    // call metaclass pre/post conditions;
    GBool _call_class_add_point_precond(int*, GPoint*) const;
    GBool _call_class_del_point_precond(int*) const;
    GBool _call_class_move_point_precond(int*, GPoint*) const;
    GBool _call_class_reset_points_precond(int, const GPoint*) const;
    void _call_class_add_point_postcond(int);
    void _call_class_del_point_postcond(int);
    void _call_class_move_point_postcond(int);
    void _call_class_reset_points_postcond();
    
    GBool _add_point(int, const GPoint&);
    GBool _move_point(int, const GPoint&);
    GBool _del_point(int);
    GBool _reset_points(int, const GPoint*);
    
    virtual GoMetaclass* m_metaclass() const;
    virtual void         m_check() const;
};


inline  GBool GoPolypoint::callAddPointPrecond(int* index,
					       GPoint* point) const
{
    return _call_class_add_point_precond(index, point);
}

inline  GBool GoPolypoint::callDelPointPrecond(int* index) const
{
    return _call_class_del_point_precond(index);
}

inline GBool GoPolypoint::callMovePointPrecond(int* index,
					       GPoint* point) const
{
    return _call_class_move_point_precond(index, point);
}

inline  GBool GoPolypoint::callResetPointsPrecond(int index,
						  const GPoint* points) const
{
    return _call_class_reset_points_precond(index, points);

}


#endif /* _GoPolypoint_h */
