/* $Id: Rectangle.h,v 1.3 1992/12/11 08:35:12 jojic Exp $ */
/*****************************************************************************
    Copyright (c) BULL S.A 1990

MODULE: Graphic Objects
FILE:   Rectangle.h
AUTHOR: Oliver Jojic, Gilles Benati
DATE:   Feb 1991

DESCRIPTION :

    This is one of the basic graphic objects, subclass of polygon.
    It is defined by to positive Number: its width and its height.
    The origin of the coord system is the upper-left corner of the rectangle,
    according to the parent coordsyst, and the edges are parallel to the axises
    i.e. seeing the rectangle as a polygon, its point array is
    [(0,0), (width, 0), (width, height). (0, height)].

    The Rectangle is subclass of Polygon. It is to implements some
    functionnalities like 'round'. But the the API is not the polygon one.
    
COPYRIGHT :    
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/
#ifndef _GoRect_h
#define _GoRect_h

#include <Go/Polygon.h>
#include <Go/mRectangle.h>

class GoRectangle : public GoPolygon {
    friend class GoRectangleMetaclass;

  public:

    GoRectangle(GNumber w = 1, GNumber h = 1);

    static GoRectangleMetaclass* metaclass();
    static GoRectangle* create(GNumber w, GNumber h);
    static GoRectangle* create() {
	return (GoRectangle*) Go::Create(GoRectangle::metaclass());
    }

    GNumber width()  const { return(points()[2].x); }
    GNumber height() const { return(points()[2].y); }
    
    virtual GBool reset(GNumber w, GNumber h);
    virtual GBool width(GNumber w);
    virtual GBool height(GNumber h);

    
    // pre/post conditions;
    void widthPrecond(GBool func(void*, const GoRectangle*, GNumber*),
		      void* closure, GAddOrRemove = GADD);
    void widthPostcond(void func(void*, GoRectangle*),
		       void* closure, GAddOrRemove = GADD);
    void heightPrecond(GBool func(void*, const GoRectangle*, GNumber*),
		       void* closure, GAddOrRemove = GADD);
    void heightPostcond(void func(void*, GoRectangle*),
			void* closure, GAddOrRemove = GADD);
    void resetPrecond(GBool func(void*, const GoRectangle*, GNumber*, GNumber*),
		      void* closure, GAddOrRemove = GADD);
    void resetPostcond(void func(void*, GoRectangle*),
		       void* closure, GAddOrRemove = GADD);

    // call preconditions in public;
    GBool callWidthPrecond(GNumber*);
    GBool callHeightPrecond(GNumber*);
    GBool callResetPrecond(GNumber* width, GNumber* height);
    
    // save/restore;
    static int	 save(const GoRectangle*, GoStream*);
    static int	 restore(GoRectangle**, GoStream*);   // creates + restores;
    static int	 readParams(GoRectangle*, GoStream*); // restores only;

  protected:

    virtual GBool m_contains(const GPoint&, GNumber maxDist = 0) const;
    virtual void m_add_clip(GRegion&) const;
    virtual void m_add_opaque_clip(GRegion&) const;
    virtual void m_draw(const GRegion&) const;
    virtual void m_wired_draw(const GCoordSys&, const GPen&, GQuality) const;

    GBool Reset(GNumber w, GNumber h);
    
  private:
    // pre/post condition flags;
    GBool _has_width_precond   :1;
    GBool _has_width_postcond  :1;
    GBool _has_height_precond  :1;
    GBool _has_height_postcond :1;
    GBool _has_reset_precond   :1;
    GBool _has_reset_postcond  :1;

    // call pre/post conditions;
    GBool _call_width_precond(GNumber*) const;
    GBool _call_height_precond(GNumber*) const;
    GBool _call_reset_precond(GNumber*, GNumber*) const;
    void _call_width_postcond();
    void _call_height_postcond();
    void _call_reset_postcond();

    // call metaclass pre/post conditions;
    GBool _call_class_width_precond(GNumber*) const;
    GBool _call_class_height_precond(GNumber*) const;
    GBool _call_class_reset_precond(GNumber*, GNumber*) const;
    void _call_class_width_postcond();
    void _call_class_height_postcond();
    void _call_class_reset_postcond();

    virtual GoMetaclass* m_metaclass() const;
    virtual void         m_check() const;

    virtual GBool reset(int, const GPoint*);           // {return GFALSE};
    virtual GBool addPoint(int, const GPoint&, GBool);  // {return GFALSE};
    virtual GBool movePoint(int, const GPoint&, GBool); // {return GFALSE};
    virtual GBool delPoint(int);                      // {return GFALSE};

    void _set_wh(GNumber w, GNumber h);
};


inline GBool GoRectangle::callWidthPrecond(GNumber* width)
{
    return _call_width_precond(width);
}

inline GBool GoRectangle::callHeightPrecond(GNumber* height)
{
    return _call_height_precond(height);
}

inline GBool GoRectangle::callResetPrecond(GNumber* width, GNumber* height)
{
    return _call_reset_precond(width, height);
}
#endif /* _GoRect_h */

