/* $Id: Text.h,v 1.8 1992/12/23 13:55:51 andre Exp $ */
/*****************************************************************************
MODULE:  GoText
FILE:    Text.h
AUTHOR:  Michel Andre
DATE:    August 91
 

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/

#ifndef		_Text_h
#define		_Text_h

#include	<Go/Textual.h>
#include	<Go/TextLn.h>
#include	<Go/mText.h>


enum GStyleValues
{
    GBACKGROUND_SHBRUSH		= (1 << 1),
    GSTRING_SHTEXTBRUSH		= (1 << 2),
    GUNDERLINE_SHPEN		= (1 << 3),
    GMAP_UNDERLINE_SEG		= (1 << 4),
    GMAP_BACKGROUND_SEG		= (1 << 5),
    GLINE_HEIGHT		= (1 << 6),
    GUNDERLINE_POSITION		= (1 << 7),
    GEXTRA_BASELINE		= (1 << 8),
};


/* Value Structure for operations (with defaults) */
class GStyle{
public:
			GStyle();
			GStyle (GStyle*);
			~GStyle();

    GBool		equiv (const GStyle*) const;
    GBool		equiv (const GoText*) const;
    void		modify (const GoText*);

    /* get functions */
    GShTextBrush* stringShTextBrush() const
	{return ((GStyle*)this)->_stringShTextBrush;}
    GShBrush* backgroundShBrush() const
	{return ((GStyle*)this)->_backgroundShBrush;}
    GShPen* underlineShPen() const
	{return ((GStyle*)this)->_underlineShPen;}
    GNumber      lineHeight() const
	{return ((GStyle*)this)->_lineHeight;}
    GNumber      underlinePosition() const
	{return ((GStyle*)this)->_underlinePosition;}
    GBool        mapBackgroundSeg() const
	{return ((GStyle*)this)->_mapBackgroundSeg;}
    GBool        mapUnderlineSeg() const
	{return ((GStyle*)this)->_mapUnderlineSeg;}
    GNumber      extraBaseline() const
	{return ((GStyle*)this)->_extraBaseline;}
    ulong        mask() const
	{return ((GStyle*)this)->_mask;}

    /* set functions */
    void	lineHeight (GNumber);
    void	extraBaseline (GNumber);
    void	underlinePosition (GNumber);
    void	mapBackgroundSeg (GBool);
    void	mapUnderlineSeg (GBool);
    void	backgroundShBrush (const GShBrush*);
    void	backgroundShBrush (ulong, const GBrushValues*);
    void	backgroundShBrush (const GBrush* br);
    void	underlineShPen (const GShPen*);
    void	underlineShPen (ulong, const GPenValues*);
    void	underlineShPen (const GPen*);
    void	stringShTextBrush (const GShTextBrush*);
    void	stringShTextBrush (ulong, const GTextBrushValues*);
    void	stringShTextBrush (const GTextBrush*);

private:
    GShTextBrush*	_stringShTextBrush;
    GShBrush*		_backgroundShBrush;
    GShPen*		_underlineShPen;
    GNumber		_lineHeight;
    GNumber		_underlinePosition;
    GBool		_mapBackgroundSeg : 1;
    GBool		_mapUnderlineSeg : 1;
    GNumber		_extraBaseline;
    ulong		_mask;
};


class GoText: public GoTextual
{
    friend class	GFormatter;
    friend class	GSimpleFormatter;
    friend class	GFrFormatter;
    friend class	GFrFormatterCtxt;
    friend class	GMCFormatter;
    friend class	GMCFormatterCtxt;
    friend class	GMBFormatter;
    friend class	GMBFormatterCtxt;
    friend class	GCircleFormatter;
    friend class	GCircleFormatterCtxt;
    friend class	GoTextSegment;
    friend class	GoTextLine;
    friend class	GoTextualClass;
    friend class	GoTextClass;
    friend class	GoTextMetaclass;

public:
    static GoTextMetaclass* metaclass();
    static GoText*	create(){
	return (GoText*) Go::Create(GoText::metaclass());
    }

		GoText();
    void	mapCursor (GBool);
    void	background (GBool, GPos = -1, GPos = -1);
    GRegion     boundariesToRegion (GPos, GPos) const;
    GPos	xToPos (int line, GNumber) const;
    GPos	xyToPos (GPoint&) const;
    GPoint	posToXy (GPos = -1) const;
    GFormatter*	formatter() const {return _formatter;}
    GBool	formatter (GFormatter*);
    GPos        begLinePos (int line = -1) const;
    GPos        endLinePos (int line = -1) const;
    int         lineNumber (int line = -1) const;
    GNumber	minWidth();
    GPos	nbLines() const;
    GBool	mapUnderlineSeg() const {return _mapUnderlineSeg;}
    GBool	mapBackgroundSeg() const {return _mapBackgroundSeg;}
    GBool	extraBaseline (GNumber, GPos = -1, GPos = -1);
    void	resize();

    // modifying shared objects methods
    void	textUnderlinePen (ulong mask, const GPenValues*, GPos = -1, GPos = -1);
    void	textUnderlinePen (const GShPen*, GPos = -1, GPos = -1);
    void	textUnderlinePen (const GPen*, GPos = -1, GPos = -1);

    void	textStringTextBrush (ulong mask, const GTextBrushValues*, GPos = -1, GPos = -1);
    void	textStringTextBrush (const GShTextBrush*, GPos = -1, GPos = -1);
    void	textStringTextBrush (const GTextBrush*, GPos = -1, GPos = -1);


    void	textBackgroundBrush (ulong mask, const GBrushValues*, GPos = -1, GPos = -1);
    void	textBackgroundBrush (const GShBrush*, GPos = -1, GPos = -1);
    void	textBackgroundBrush (const GBrush*, GPos = -1, GPos = -1);


    void	highlight (const GStyle&, GPos = -1, GPos = -1);

    /* stream methods */
    static int  save (const GoText*, GoStream*);
    static int  restore (GoText**, GoStream*);
    static int  readParams (GoText*, GoStream*);

    /* subGo creation and access methods */
    GBool	addGo (Go*, GPos);
    Go*		subGo (const GPoint& absp);
    void	selectUnselectSubGo (const GPoint& absp);

    /* extract/merge */
    GoText*	extract (GPos, GPos);
    GBool	merge (GoText*, GPos = 0);

protected:
    void	m_dup(const Go*, Go*, Go*, GBool, GBool);
    GoMetaclass*m_metaclass() const;
    void	m_draw (const GRegion&) const;
    GBool       m_contains (const GPoint& absp, GNumber maxDistance = 0) const;
    void	m_add_clip (GRegion&) const;
    void	m_add_opaque_clip (GRegion&) const;

    GoTextSegment*	posToSeg (GPos) const;
    GoTextSegment*	lastSeg() const;
    void		shiftPos (GoTextSegment*, GPos);
    GNumber     	_Width() const;
    GNumber		linesHeightSum();

    void        _setString (const char*, GPos);
    void        _insertChar (char, GPos);
    void        _insertString (const char*, GPos, GPos);
    void        _deleteChar (char, GPos);
    void        _deleteString (const char*, GPos, GPos);
    void        _replaceChar (char, GPos);
    void        _replaceString (GPos, GPos, char*, GPos);
    void        _presentation (GPresentation);
    void	createPortion (GPos,  GPos);
    void	_lineH (GNumber, GPos = -1, GPos = -1);
    void	_underlinePos (GNumber, GPos = -1, GPos = -1);
    void	_underline (GBool, GPos = -1, GPos = -1);

private:
    GFormatter*	_formatter;
    GoTextLine*	_firstLine;
    GoTextLine*	_lastLine;
    GBool	_mapUnderlineSeg;
    GBool	_mapBackgroundSeg;
    friend void _modifyStringTextBrush (void*, GTextBrush*, ulong);
    friend void _modifyUnderlinePen (void*, GPen*, ulong);
    GBool	_emptySegment (GPos) const;
};

#endif			/* _Text_h */
