/* $Id: TextDefs.h,v 1.5 1992/10/26 11:25:10 andre Exp $ */
/*****************************************************************************
MODULE:  GoText
FILE:    TextDefs.h
AUTHOR:  Michel Andre
DATE:    August 91
 

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/

#include <Go/Tools/Number.h>
#include <Go/Tools/Bool.h>


#ifndef			_TextDefs_h
#define			_TextDefs_h


/* to avoid casting */
#define GSEG(i)		((GoTextSegment*)i)
#define GLINE(i)	((GoTextLine*)i)
#define GTEXT(i)	((GoText*)i)


/* defines used for Textual mask */
#define    GBACKGROUND		(1 << 1)
#define    GSTRING		(1 << 2)
#define    GUNDERLINE		(1 << 3)
#define    GCURSOR		(1 << 4)


/* justification style values */
typedef enum		{GLEFT_JUSTIFICATION, GCENTER_JUSTIFICATION,
			 GRIGHT_JUSTIFICATION, GLR_JUSTIFICATION} GJustification;

/* presentation style values */
typedef enum		{GNO_PRESENTATION, GLOWER_PRESENTATION,
			 GUPPER_PRESENTATION, GTITLE_PRESENTATION} GPresentation;

/* search directions */
typedef enum		{GTEXT_FORWARD, GTEXT_BACKWARD} GTextDirection;

/* margin sides */
typedef enum		{GLEFT_SIDE_MARGIN, GRIGHT_SIDE_MARGIN, GTOP_SIDE_MARGIN, GBOTTOM_SIDE_MARGIN} GMarginSide;


/* languages used for the word hyphenation */
typedef enum		{GGERMAN, GSPANISH, GFRENCH, GENGLISH,
			 GITALIAN, GPORTUGUESE, GNO_LANGUAGE} GLanguage;


typedef int		GPos;


/* structure used for GINSERT_CHAR_NOTIFY, GDELETE_CHAR_NOTIFY, GREPLACE_CHAR_NOTIFY */

typedef struct
{
    char	Char;
    GPos	pos;
}GCharStruct;


/* structure used for GREPLACE_STRING_NOTIFY, GSET_STRING_NOTIFY, GINSERT_STRING_NOTIFY,
GDELETE_STRING_NOTIFY */

typedef struct
{
    const char*	str;
    GPos	len;
    GPos	pos;		/* used for GINSERT_STRINGNOTIFY and GREPLACE_STRING_NOTIFY */
    GPos	oldStringSize;  /* used for GREPLACE_STRING_NOTIFY only */
} GStringStruct;


typedef enum{
    GSET_STRING_NOTIFY,
    GINSERT_CHAR_NOTIFY,
    GDELETE_CHAR_NOTIFY,
    GINSERT_STRING_NOTIFY,
    GDELETE_STRING_NOTIFY,
    GREPLACE_CHAR_NOTIFY,
    GREPLACE_STRING_NOTIFY,
    GPRESENTATION_CHANGE_NOTIFY
} GModifyValueType;


typedef struct{
    GModifyValueType type;
    union
	{
	    GCharStruct		charValue;
	    GStringStruct	stringValue;
	    GPresentation	presentation;
	}
#ifdef    _TEXTDEFS_H_FOR_C
    value
#endif
    ;

} GModifyValueStruct;


typedef enum{
    GJUSTIFICATION_CHANGE_NOTIFY,
    GINTERLINE_CHANGE_NOTIFY,
    GUNDERLINE_POSITION_CHANGE_NOTIFY,
    GUNDERLINE_SET_NOTIFY,
    GLINEHEIGHT_CHANGE_NOTIFY,
    GLEFT_EDGE_CHANGE_NOTIFY,
    GRIGHT_EDGE_CHANGE_NOTIFY,
    GTOP_EDGE_CHANGE_NOTIFY,
    GBOTTOM_EDGE_CHANGE_NOTIFY,
    GEXTRA_BASELINE_CHANGE_NOTIFY,
    GMARGIN_SIDE_CHANGE_NOTIFY
} GModifyAttributeType;

/* structure used for GMARGIN_SIDE_CHANGE_NOTIFY */

typedef struct
{
    GNumber	margin;
    GMarginSide	marginSide;
}GMarginStruct;


typedef struct
{
    GModifyAttributeType type;
    GPos	firstPosition;
    GPos	lastPosition;
    union
	{
	    GJustification	justification;
	    GNumber		interline;
	    GNumber		underlinePosition;
	    GNumber		lineHeight;
	    GBool		underlineSet;
	    GNumber		baseline;
	    GMarginStruct	marginValue;
	}
#ifdef   _TEXTDEFS_H_FOR_C
    attribute
#endif
    ;

} GModifyAttributeStruct;

#endif			/* _TextDefs_h */
