/* $Id: TextSg.h,v 1.10 1992/12/23 14:08:00 andre Exp $ */
/*****************************************************************************
MODULE:  GoText
FILE:    TextSg.h
AUTHOR:  Michel Andre
DATE:    August 91
 

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/

#ifndef		_TextSg_h
#define		_TextSg_h

#include	<Go/Go.h>
#include	<Go/Tools/Brush.h>
#include	<Go/Tools/TextBrush.h>
#include	<Go/Tools/Pen.h>
#include	<Go/TextDefs.h>


class GoTextSegment : public Go
{
  public:
    static GoMetaclass*		metaclass();
    static GoTextSegment* 	create();

    // stream functions
    static int          save (const GoTextSegment*, GoStream*);
    static int          restore (GoTextSegment**, GoStream*);
    static int          readParams (GoTextSegment*, GoStream*);

    friend void		GSeg_split (GoTextSegment*, GPos);
    friend GBool	GSeg_append (GoTextSegment*);

    GNumber             normalSpace();
    GNumber             minSpace (GNumber shrinkLevel)
                            {return normalSpace() * (1 - shrinkLevel);}
    GNumber             maxSpace (uchar strechLevel)
                            {return normalSpace() * strechLevel;}
    GBool		equiv (GoTextSegment*) const;
    GBool		indented (const class GoText* = NULL) const;

    const GShTextBrush*	stringShTextBrush() const;
    const GShBrush*	backgroundShBrush() const;
    const GShPen*	underlineShPen() const;
    GNumber		lineHeight() const;
    GNumber		underlinePosition() const;
    GNumber		extraBaseline() const;
    GBool		mapUnderlineSeg() const;
    GBool		mapBackgroundSeg() const;

  protected:
    void	 	m_dup(const Go*, Go*, Go*, GBool, GBool);
    GoMetaclass* 	m_metaclass() const;
    void	 	m_add_clip (GRegion&) const;
    virtual void	m_draw (const GRegion&) const;
    virtual GBool	m_contains (const GPoint& , GNumber) const;

    GBool		contains (GoTextSegment&) const;
    void		underlinePosition (GNumber);
    GoTextSegment*	myNext() const;
    GoTextSegment*	myPrev();
    virtual void	updateWidth();
    virtual void	updateHeight();
    void		updateHighlight();
			GoTextSegment();
			~GoTextSegment();

    class GStyle*	_att;
    GPos		_firstPos;
    GPos		_lastPos;
    GNumber		_width;
    GNumber		_height;
    GBool              	_hyphened : 1;

private:
    virtual GAssoc*      d_get_assoc() const {return NULL;}
    virtual Go*          d_get_parent() const {return _parent;}
    virtual Go*          d_get_previous() const;
    virtual Go*          d_get_next() const {return _next;}
    virtual Go*          d_get_first_child() const {return NULL;}
    virtual GBool        d_store_children_array() const {return GFALSE;}
    virtual void         d_set_previous (const Go*){}
    virtual void         d_set_next (const Go* go) {_next = (Go*)go;}
    virtual void         d_remove_child (const Go*, int);
    virtual void         d_add_child (const Go*, int);
    virtual void         d_set_tree (const Go*, const Go*, const Go*);
    virtual GBool        d_store_abscs() const {return GFALSE;}
    virtual GBool        d_store_relcs() const {return GTRUE;}
    virtual const GCoordSys& d_get_relcs (const GCoordSys*) const;
    virtual void         d_set_coordsys (const GCoordSys& relcs,
		         const GCoordSys&,const GCoordSys&){_origin = relcs.O;}

    /*** data ***/
    Go* 		_parent;
    Go* 		_next;
    GPoint		_origin;

    friend void         _modifyStringTextBrush (void*, GTextBrush*, ulong);
    friend void         _modifyUnderlinePen (void*, GPen*, ulong);

    friend class	GoTextSegmentMetaclass;
    friend class	GFormatter;
    friend class	GSimpleFormatter;
    friend class	GFrFormatter;
    friend class	GFrFormatterCtxt;
    friend class	GMCFormatter;
    friend class	GMCFormatterCtxt;
    friend class	GMBFormatter;
    friend class	GMBFormatterCtxt;
    friend class	GCircleFormatter;
    friend class	GCircleFormatterCtxt;
    friend class	GoTextLine;
    friend class	GoText;
};


INLINE GoTextSegment* GoTextSegment::create(){
    return (GoTextSegment*) Go::Create(GoTextSegment::metaclass());
}


#endif			/* _TextSg_h */
