/* $Id: Textual.h,v 1.6 1992/12/11 08:35:26 jojic Exp $ */
/*****************************************************************************
MODULE:  GoText
FILE:	 Textual.h
AUTHOR:  Michel Andre
DATE:    August 91
 
  See also mTextual.h for Go metaclass informations.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France

  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.

  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/

#ifndef		_GoTextual_h
#define		_GoTextual_h

#include	<Go/GoStdDS.h>
#include	<Go/Tools/Brush.h>
#include	<Go/Tools/TextBrush.h>
#include	<Go/Tools/Pen.h>
#include	<Go/TextDefs.h>
#include	<Go/TextList.h>
#include	<Go/mTextual.h>


/**********************
*   class GoTextual
**********************/

class GoTextual: public GoStdDS {
    friend class GoTextualMetaclass;

public:

    static GoTextualMetaclass* metaclass();

    // insertion functions
    GBool	insertChar (char, GPos = -1);
    GBool	insertString (const char*, GPos, GPos = -1);

    // deletion functions
    GBool	deleteChar (GPos = -1);
    GBool	deleteString (GPos, GPos = -1);

    // string functions
    GBool	setString (const char*, GPos);
    const char*	getString() const {return _string;} 
    GPos	length() const {return (strlen (_string));}
    GPos	containChar (char, GPos = 0, GTextDirection = GTEXT_FORWARD, GBool caseMatching = GFALSE) const;
    GPos	containString (char*, GPos, GPos = 0, GTextDirection = GTEXT_FORWARD, GBool caseMatching = GFALSE) const;
    char	operator[](GPos) const;
    GBool	replaceChar (char, GPos = -1);
    GBool	replaceString (GPos, GPos, char*, GPos);


    // attributes functions
    GBrush&     backgroundBrush() const{return ((GoTextual*)this)->_backgroundBrush;}
    GBrush&     selectionBrush() const{return ((GoTextual*)this)->_selectionBrush;}
    GTextBrush&	stringTextBrush() const{return ((GoTextual*)this)->_stringTextBrush;}
    GPen&	cursorPen() const{return ((GoTextual*)this)->_cursorPen;}
    GPen&	underlinePen() const{return ((GoTextual*)this)->_underlinePen;}

    virtual GBool	presentation (GPresentation);
    GPresentation 	presentation() const {return _present;}
    GBool	lineHeight (GNumber, GPos = -1, GPos = -1);
    GNumber	lineHeight() const {return _lineHeight;}
    GNumber	lineHeightMax() const {return _lineHeightMax;}
    GBool	underlinePosition (GNumber, GPos = -1, GPos = -1);
    GBool	underline (GBool, GPos = -1, GPos = -1);
    GNumber     underlinePosition() const {return _underlinePosition;}
    virtual GBool	justification (GJustification);
    GJustification	justification() const {return _justification;}
    GBool	interline (GNumber);
    GNumber     interline() const {return _interline;}
    GBool	margin (GNumber, GMarginSide);
    GNumber	margin (GMarginSide) const;


    // cursor functions
    void	cursorUp();
    void	cursorDown();
    void	cursorLeft();
    void	cursorRight();
    void	cursorBegText();
    void	cursorEndText();
    void	cursorBegLine();
    void	cursorEndLine();
    virtual GBool	currentPos (GPos);
    GPos	getCurrentPos() const {return _currentPos;}


    // tools functions
    virtual GPos	nbLines() const;
    virtual int	        lineNumber (GPos = -1) const;
    virtual GPos	xToPos  (int, GNumber) const {return 0;}
    virtual GPos	xyToPos  (GPoint&) const {return 0;}
    virtual GPoint	posToXy  (GPos = -1) const {return *((GPoint*)NULL);}
    virtual GPos	begLinePos (int = -1) const;
    virtual GPos	endLinePos (int = -1) const;
    void		wordBoundaries (GPos, GPos&, GPos&);
    void		lineBoundaries (GPos, GPos&, GPos&);
    virtual GRegion	boundariesToRegion (GPos, GPos) const;
    GNumber     	width() {return _width;}
    GNumber     	height() {return _height;}
    virtual void	resize(){};


    // map functions
    void		mapString (GBool set) {_updateMask (GSTRING, set);}
    virtual void	mapBackground (GBool set) {_updateMask (GBACKGROUND, set);}
    virtual void	mapCursor (GBool set) {_updateMask (GCURSOR, set);}
    virtual void	mapUnderline (GBool set) {_updateMask (GUNDERLINE, set);}
    GBool		stringMapped() const {return _mask & GSTRING;}
    GBool		backgroundMapped() const {return _mask & GBACKGROUND;}
    GBool		cursorMapped() const {return _mask & GCURSOR;}
    GBool		underlineMapped() const {return _mask & GUNDERLINE;}


    // selection operations
    GBool	selectPortion (GPos, GPos, GBool);
    GPos	firstSelected();
    GPos	lastSelected();
    char*	selectedString();
    GTextList&	selectionList() const {return ((GoTextual*)this)->_selectionList;}


    // Pre and Post Conditions

    /* mofify value */
    void	modifyValuePrecond (GBool (*)(void*, const GoTextual*, GModifyValueStruct*),
                void* closure, GAddOrRemove = GADD);
    void	modifyValuePostcond (void (*)(void*, GoTextual*, GModifyValueStruct*),
		void* closure, GAddOrRemove = GADD);

    /* current position change */
    void	currentPosPrecond (GBool (*)(void*, const GoTextual*, GPos),
                void* closure, GAddOrRemove = GADD);
    void	currentPosPostcond (void (*)(void*, GoTextual*, GPos),
		void* closure, GAddOrRemove = GADD);

    /* attribute */
    void        modifyAttributePrecond (GBool (*)(void*, const GoTextual*, GModifyAttributeStruct*),
                void* closure, GAddOrRemove = GADD);
    void        modifyAttributePostcond (void (*)(void*, GoTextual*, GModifyAttributeStruct*),
                void* closure, GAddOrRemove = GADD);

    /* selection */
    void        selectionPrecond (GBool (*)(void*, const GoTextual*, GPos, GPos),
                void* closure, GAddOrRemove = GADD);
    void        selectionPostcond (void (*)(void*, GoTextual*, GPos, GPos),
                void* closure, GAddOrRemove = GADD);

    // GoStream functions
    static int	save(const GoTextual*, GoStream*);
    static int	readParams(GoTextual*, GoStream*);

protected:
    ulong	_mask;
    GPresentation	_present;
    GNumber     _underlinePosition;
    GNumber     _lineHeight;
    GNumber     _lineHeightMax;
    GBrush      _backgroundBrush;
    GBrush      _selectionBrush;
    GTextBrush  _stringTextBrush;
    GPen        _underlinePen, _cursorPen;
    char*	_string;
    GPos        _size, _currentPos;
    GTextList	_selectionList;
    GNumber	_width, _height;
    GoTextualCondFlags	_flags;
    GNumber     _interline;
    GJustification      _justification;
    GNumber	_leftMargin;
    GNumber	_rightMargin;
    GNumber	_topMargin;
    GNumber	_bottomMargin;
    void	_allocString (GPos);

    virtual void m_dup (const Go*, Go*, Go*, GBool, GBool);
    virtual GoMetaclass* m_metaclass() const;
    virtual void m_add_clip (GRegion&) const;
    virtual void m_add_opaque_clip (GRegion&) const;
    void     m_wired_draw (const GCoordSys& cs, const GPen&, GQuality) const;
    virtual GBool m_contains(const GPoint& absp, GNumber maxDistance = 0) const;
    void	_updateMask (ulong, GBool);
    void	_updatePresentation (GPresentation);
    GBool	_call_modify_value_precond (GModifyValueStruct*);
    void	_call_modify_value_postcond (GModifyValueStruct*);
    GBool	_call_current_pos_precond (GPos);
    void	_call_current_pos_postcond (GPos);
    GBool	_call_modify_attribute_precond (GModifyAttributeStruct*);
    void	_call_modify_attribute_postcond (GModifyAttributeStruct*);
    GBool	_call_selection_precond (GPos, GPos);
    void	_call_selection_postcond (GPos, GPos);
    static void	_select (void*, Go*);
    virtual GNumber	_Width() const {return _width;}

    virtual void	_setString (const char*, GPos){};
    virtual void	_insertChar (char, GPos){};
    virtual void	_insertString (const char*, GPos, GPos){};
    virtual void	_deleteChar (char, GPos){};
    virtual void	_deleteString (const char*, GPos, GPos){};
    virtual void	_replaceChar (char, GPos){};
    virtual void	_replaceString (GPos, GPos, char*, GPos){};
    virtual void	_presentation (GPresentation);
    virtual void	_underline (GBool, GPos, GPos){}
    virtual void	_underlinePos (GNumber, GPos, GPos){}
    virtual void	_lineH (GNumber, GPos, GPos){}

		GoTextual();
		~GoTextual();
};

#endif		/* _GoTextual_h */
