/* $Id: Array.h,v 1.1.1.1 1992/03/18 09:54:35 uep Exp $ */
/*****************************************************************************
MODULE:  Tools
FILE:    Array.h
AUTHOR:  Philippe Smadja
DATE:    Dec 90

DESCRIPTION:
         Implementation of an array of elements. The elemsize
	 is constant.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _Array_h
#define _Array_h

#include <Go/machdef.h>
#include <Go/Tools/Bool.h>

/**
  [--------------|||||||||||||||||||||--------]
  ^              ^
  |              |
  _data          _firstIndex
                 .......  ............
		        \/
		       _nelems

  .......................  ...................
                         \/
		      _nelemsAvailable
   **/

#define GLAST_INDEX (_firstIndex+_nelems)
#define GNB_AVAILABLE(n_to_add) (((_nelems+(n_to_add))+ _incr -1)/_incr*_incr)

class GArray {
  public:
    /**
      methods
      **/
    GArray(int elemsize = sizeof(void*), int incr = 1);
    GArray(const void* value, int nelems = 1, int elemsize = sizeof(void*), int incr = 1);
    ~GArray();

    /**
      ADD:-1 means: at the end  of the array.
      ie: index = 0; the element 0 is replaced by "nelems" at
      index 0. the other value are  shifted of "nelems".
      **/
    void add(const void* value, uint nelems = 1, int index = -1);

    /**
      REM:  -1 means: the last element
      **/
    void rem (int index = -1, uint nelems_to_free = 1); 
    void rem (const void* value, GBool only_first_element = GTRUE);
    inline void  reset() { rem(0,_nelems); }
    /**
      ACCES.
      **/
    inline int  incr() { return _incr; }
    inline void* data() const {
	return (void*) (_data + _firstIndex*_elemsize);
    }
    inline int   elemsize() const { return _elemsize;}
    inline int   nelems()  const { return _nelems;}

  protected:
    uchar*  _data;
    uint    _elemsize;
    uint    _nelems;
    uint    _nelemsAvailable;
    uint    _incr;
    uint    _firstIndex;

};


struct GPointerArray: public GArray {
    GPointerArray(int incr = 1): GArray(sizeof(void*), incr) {}
    GPointerArray(void* value, int incr = 1): GArray(&value,1,sizeof(void*), incr){}
    GPointerArray(void* values[], int nelems, int incr): GArray(values, nelems, sizeof(void*), incr){}


    /**
      ADD: -1 means at the end of the array
      **/
    void add(const void* value, int index = -1);
    inline void add(const void* values[], int nelems_to_add, int index) {
      GArray::add(values, nelems_to_add, index);
    }
    
    /**
      REM: -1 means the last element
      **/
    inline void rem(int index = -1, int nelems_to_free = 1) {
      GArray::rem(index, nelems_to_free);
    }
    
    void rem(const void* value, GBool only_first_element = GTRUE);
    /**
      ACCES
      **/
    void** data() const  { return (void**)GArray::data(); }
    void*  data(int index) const { return  (void*) *(ulong*)(_data + (index+_firstIndex)*_elemsize);}
};

#endif /* _Array_h */
