/* $Id: Color.h,v 1.1.1.1 1992/03/18 09:55:11 uep Exp $ */
/*****************************************************************************
MODULE:  Tools
FILE:    Color.h
AUTHOR:  Jacques Davy
DATE:    November 1990

DESCRIPTION:
    This file describes the Color object.

    The Color object gives a regular interface to the shared object
    ShColor (see ShColor.h) and it allows to set some pre/post conditions.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _Color_h
#define _Color_h

#include <Go/Tools/ShColor.h>
#include <Go/Tools/Assoc.h>
#include <Go/Tools/Callback.h>

/*
 * Condition flags
 */
struct GColorCondFlags
{
    GBool _has_modify_precond : 1;
    GBool _has_modify_postcond : 1;
    GBool _has_destroy_postcond : 1;

    GColorCondFlags()
    {
	_has_modify_precond  = GFALSE;
	_has_modify_postcond  = GFALSE;
	_has_destroy_postcond  = GFALSE;
    }
};

/*
 * GColor class definition
 */
class GColor
{
  public:
    GColor ();
    GColor (const GColor* color);
    GColor (const GShColor* sh_color);
    GColor (ulong mask, const GColorValues* values, const char* name = NULL);
    GColor (const GRgb* rgb, const char* name = NULL);
    GColor (uchar gray, const char* name = NULL);
    GColor (const class GShPattern* halftone, const char* name = NULL);
    GColor (const char* name);
    ~GColor ();

    // Getting values
    inline const GShColor* shColor() const { return _sh_color; }
    inline const GColorValues* values() const { return _sh_color->values(); }

    inline const char* name() const { return _sh_color->name(); }
    inline GBool isNamed() const { return _sh_color->isNamed(); }

    inline const GRgb* rgb() const { return _sh_color->rgb(); }
    inline int gray () const { return _sh_color->gray(); }
    inline const class GShPattern* halftone () const { return _sh_color->halftone(); }

    // Modifying methods
    void modify (ulong mask, const GColorValues* values);
    void modify (const GShColor*);
    void modify (const char* name);

    void rgb (const GRgb* rgb);
    void gray (uchar gray);
    void halftone (const class GPattern* halftone);
    void halftone (const class GShPattern* halftone);
    void halftone (const char* name);

    // Pre and Post Conditions
    void modifyPrecond (GBool (*)(void*, const GColor*, ulong&, GColorValues*),
			const void* closure,
			GAddOrRemove = GADD) const;
    void modifyPostcond (void (*)(void*, GColor*, ulong), const void* closure,
			 GAddOrRemove = GADD) const;
    void destroyPostcond (void (*)(void*, const GColor*), const void* closure,
			  GAddOrRemove = GADD) const;
    // friends
    friend void Call_call_modify_postcond (GColor* This, const GShColor*, ulong); 

  private:
    GShColor* _sh_color;


  protected:
    GColorCondFlags _flags;
    GAssoc _assoc;

    int _addCallback (const char* type, GCBProc proc, const void* closure) const;
    int _remCallback (const char* type, GCBProc proc, const void* closure) const;

    GBool _call_modify_precond (ulong& modify_mask, GColorValues*);
    void _call_modify_postcond (ulong modify_mask);
    void _call_destroy_postcond ();

};

#endif /* _Color_h */
