/* $Id: CoordSys.h,v 1.3 1992/05/26 16:34:14 benati Exp $ */
/*****************************************************************************
MODULE:  Tools
FILE:    CoordSys.h
AUTHOR:  Olivier Jojic
DATE:    jan 90

DESCRIPTION:

The CoordSys struct implements a coordinate system.
A coordinate system is compound of an origin O , an X axis vector I and
an Y axis vector J.

The methods "from()" and "to()" transform a Point, Vector, Region or
CoordSys from the absolute coordinate system to this one.
Examples:
"cs" is a CoordSys.
    - You have an absolute point "absp".
      The same point in "cs" coordinates is:   relp = cs->from(absp);
    - You have a relative vector "relv".
      The same absolute vector is:             absv = cs->to(relv);

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/

#ifndef _CoordSys_h
#define _CoordSys_h

#include <Go/Tools/Vector.h>
#include <Go/Tools/Point.h>
#include <Go/Tools/Region.h>

struct GCoordSys {
    GPoint  O;
    GVector I, J;

    GCoordSys();
    GCoordSys(const GPoint& o, const GVector& i, const GVector& j);
    GCoordSys(const GCoordSys&);
    GCoordSys(const GPoint&, const GPoint&);

    friend GBool operator == (const GCoordSys&, const GCoordSys&);
    friend GBool operator != (const GCoordSys& a, const GCoordSys& b)
    	{ return !(a == b); }
    GCoordSys operator ~() const;
    
    GBool isOrthogonal() const;

    GNumber deter() const { return I.x * J.y - I.y * J.x; }

    void rotate(const GPoint& center, GNumber angle);
    GNumber angle() const { return I.angle(); }
    void angle(GNumber absoluteAngle);

    GVector from(const GVector&) const;
    GVector to(const GVector&) const;

    GBox    from(const GBox&) const;
    GBox    to(const GBox&) const;
    
    GPoint from(const GPoint&) const;
    GPoint to(const GPoint&) const;
    
    GRegion from(const GRegion&) const;
    GRegion to(const GRegion&) const;
    
    GCoordSys from(const GCoordSys&) const;
    GCoordSys to(const GCoordSys&) const;

    GCoordSys& invert();

    friend GCoordSys operator * (const GCoordSys& a, const GCoordSys& b)
    	{ return a.to(b); }
};


const GCoordSys& GUnitCoordSys();

#endif /* _CoordSys_h */
