/* $Id: FileMedium.h,v 1.1.1.1 1992/03/18 09:55:39 uep Exp $ */
/******************************************************************************
MODULE: Tools
FILE:	FileMedium.h
AUTHOR:	Philippe Smadja
DATE:   Jun 91

DESCRIPTION:
   The Medium class provides an interface to read/write  from/to "somethink" called
   here Medium.  The FileMedium class  is subclass of Medium. It provides the
   input/output virtual method and specific constructors
   
COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
 
#ifndef _FileMedium_h
#define _FileMedium_h

#include <Go/Tools/Medium.h>
#include <Go/Tools/Bool.h>
#include <stdio.h>


class GFileMedium  : public GMedium {
  public:

    GFileMedium();
    GFileMedium(FILE* file);
    GFileMedium(const char* filename, const char* type = "r");
    inline FILE* file() { return _file; } ;

    ~GFileMedium();

    int open(const char* filename, const char* type); /** for type cf fopen()
						       **/
    void flush();
    void close();


    // OUPUT;
    
    virtual long write(const char*, long);
    virtual int  mputc(char c);
    virtual int  mputs(const char* s);
    virtual int  mputback(char);

    // INPUT;
    
    virtual long read(char*, long);
    virtual int  mgetc();
    virtual char*  mgets(char* s, ulong max_size);
    virtual int  readuntil(char stop);

    // POSITION;
	
    virtual int mseek(long offset, GMSeekType st = GMSEEK_FROM_BEGINNING);
    virtual long mtell();
	


  protected:
    uchar _fileMustBeClosed : 1,
          _opened : 1;

    FILE* _file;
    int _commonOpen(const char* filename, const char* type);
};

#endif /* _FileMedium_h */

