/* $Id: Font.h,v 1.1.1.1 1992/03/18 09:55:47 uep Exp $ */
/*****************************************************************************
MODULE:  Tools
FILE:    Font.h
AUTHOR:  Jacques Davy
DATE:    November 1990

DESCRIPTION:
    This file describes the Font object.

    The Font object gives a regular interface of the shared object ShFont
    (see ShFont.h). It also allows to set some pre/post conditions.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _Font_h
#define _Font_h

#include <Go/Tools/ShFont.h>
#include <Go/Tools/Assoc.h>
#include <Go/Tools/Callback.h>

/*
 * Condition flags
 */
struct GFontCondFlags
{
    GBool _has_modify_precond : 1;
    GBool _has_modify_postcond : 1;
    GBool _has_destroy_postcond : 1;

    GFontCondFlags() {
	_has_modify_precond  = GFALSE;
	_has_modify_postcond  = GFALSE;
	_has_destroy_postcond  = GFALSE;
    }
};

/*
 * GFont class definition
 */
class GFont
{
  public:
    GFont ();
    GFont (const GFont* font);
    GFont (const GShFont* sh_font);
    GFont (ulong mask, const GFontValues* values, const char* name = NULL);
    GFont (const char* name);
    ~GFont ();

    // Getting values
    inline const GShFont* shFont() const { return _sh_font; }
    inline const GFontValues* values() const { return _sh_font->values(); }

    inline const char* name() const { return _sh_font->name(); }
    inline GBool isNamed() const { return _sh_font->isNamed(); }

    inline char* fullName () const { return _sh_font->fullName(); }
    inline const char* registry () const {return _sh_font->registry(); }
    inline const char* family () const { return _sh_font->family(); }
    inline const char* weight () const { return _sh_font->weight(); }
    inline const char* slant () const { return _sh_font->slant(); }
    inline const char* width () const { return _sh_font->width(); }
    inline const char* addStyle () const { return _sh_font->addStyle(); }
    inline const char* spacing () const { return _sh_font->spacing(); }
    inline const char* charset () const { return _sh_font->charset(); }
    inline const char* charcode () const { return _sh_font->charcode(); }

    // Getting string metrics (see GoInterf.h)

    // Modifying methods
    void modify (ulong mask, const GFontValues* values);
    void modify (const GShFont* font);
    void modify (const char* name);

    // Pre and Post Conditions
    void modifyPrecond (GBool (*)(void*, const GFont*, ulong&, GFontValues*),
			const void* closure,
			GAddOrRemove = GADD) const;
    void modifyPostcond (void (*)(void*, GFont*, ulong),
			 const void* closure,
			 GAddOrRemove = GADD) const;
    void destroyPostcond (void (*)(void*, const GFont*),
			  const void* closure,
			  GAddOrRemove = GADD) const;
    // friends
    friend void Call_call_modify_postcond (GFont* This, const GShFont*, ulong); 

  private:
    GShFont* _sh_font;


  protected:
    GFontCondFlags _flags;
    GAssoc _assoc;

    int _addCallback (const char* type, GCBProc proc, const void* closure) const;
    int _remCallback (const char* type, GCBProc proc, const void* closure) const;

    GBool _call_modify_precond (ulong& modify_mask, GFontValues*);
    void _call_modify_postcond (ulong modify_mask);
    void _call_destroy_postcond ();
};

#endif /* _Font_h */
