/* $Id: Medium.h,v 1.1.1.1 1992/03/18 09:56:32 uep Exp $ */
/******************************************************************************
MODULE: Tools
FILE:	Medium.h
AUTHOR:	Philippe Smadja
DATE:   Jun 91

DESCRIPTION:
   The Medium class provides an interface to read/write  from/to "something" called
   here Medium. The effective medium (a subclass) must implemente the effective
   input/output.
   
COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
 
#ifndef _Medium_h
#define _Medium_h

#include <Go/machdef.h>

#define GEOM (-1)

enum GMSeekType {
    GMSEEK_FROM_BEGINNING = 0,
    GMSEEK_FROM_CURRENT,
    GMSEEK_FROM_END
};

class GMedium {

  public:

    virtual ~GMedium();
    
    // OUPUT;
    
    virtual long write(const char*, long) = NULL;
    /** returns the number of bytes
      effectively written, 0 on error.
      **/
    
    virtual int mputc(char c) = NULL;
    /**
      write 1 char. return c or -1 on error.
      **/
    
    virtual int mputs(const char* s) = NULL;
    /**
      write a null terminated string, the '\0' is not written
      It returns 0 if ok  and  -1 on error.
      **/
    
    virtual int mputback(char) = NULL;
    /**
      returns 1 or 0 on error, It insert the character into the medium
      before the current read/write offset. cf mungetc().
      **/

    // INPUT;
    
    virtual long read(char*, long) = NULL;
    /**
      returns the number of bytes effectively read, EOM on error.
      **/
    
    virtual int  mgetc() = NULL;
    /**
      returns the current char, GEOM on error.
      **/
    
    virtual char*  mgets(char* s, ulong max_size) = NULL;
    /**
      read characters from the medium into the array pointed to by s until
      size-1 characters are read, or a '\n'  or a end-of-medium is encounted
      The string is then terminated with a null character.
      It returns  the parameter s on completion else NULL.
      **/

    int  mungetc(char);
    /**
      another interface for mputback(char).
      **/
    
    virtual int readuntil(char stop);
    /**
      read until character c is met and point on the following character.
      **/
    

    // POSITION;
	
    virtual int mseek(long offset,GMSeekType st = GMSEEK_FROM_BEGINNING) = NULL;
    /**
      set the current read/write offset the 'offset' according the SeekType
      It returns non zero on error.
      **/

    virtual long mtell() = NULL;
    /**
      returns the current read/write offset: -1 on error.
      **/

  private:
    /**
      FOR COMPATIBILITY with compiler which does not
      accept empty data class
      **/
    char unused;
};


inline int GMedium::mungetc(char c)  { return mputback(c);};

#endif /* _Medium_h */
