/* $Id: Pattern.h,v 1.1.1.1 1992/03/18 09:56:41 uep Exp $ */
/*****************************************************************************
MODULE:  Tools
FILE:    Pattern.h
AUTHOR:  Jacques Davy
DATE:    November 1990

DESCRIPTION:
    This file describes the Pattern object.

    The Pattern object gives a regular interface to the shared object
    ShPattern (see ShPattern.h) and it allows to set some pre/post conditions.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _Pattern_h
#define _Pattern_h


#include <Go/Tools/ShPattern.h>
#include <Go/Tools/Assoc.h>
#include <Go/Tools/Callback.h>

/*
 * Condition flags
 */
struct GPatternCondFlags
{
    GBool _has_modify_precond : 1;
    GBool _has_modify_postcond : 1;
    GBool _has_destroy_postcond : 1;

    GPatternCondFlags() {
	_has_modify_precond  = GFALSE;
	_has_modify_postcond  = GFALSE;
	_has_destroy_postcond  = GFALSE;
    }
};

/*
 * GPattern class definition
 */
class GPattern
{
  public:
    GPattern ();
    GPattern (const GPattern* pattern);
    GPattern (const GShPattern* sh_pattern);
    GPattern (ulong mask, const GPatternValues* values, const char* name = NULL);
    GPattern (const char* name);
    ~GPattern ();

    // Getting values
    inline const GShPattern* shPattern() const { return _sh_pattern; }
    inline const GPatternValues* values() const { return _sh_pattern->values(); }

    inline const char* name() const { return _sh_pattern->name(); }
    inline GBool isNamed() const { return _sh_pattern->isNamed(); }

    inline const GShPixelArray* pixelArray () const { return _sh_pattern->pixelArray(); }
    inline const GShColormap* colormap () const { return _sh_pattern->colormap(); }
    inline GNumber width () const { return _sh_pattern->width(); }
    inline GNumber height () const { return _sh_pattern->height(); }

    // Modifying methods
    void modify (ulong mask, const GPatternValues* values);
    void modify (const GShPattern*);
    void modify (const char* name);

    // Pre and Post Conditions
    void modifyPrecond (GBool (*)(void*, const GPattern*, ulong&,
				  GPatternValues*),
			const void* closure,
			GAddOrRemove = GADD) const;
    void modifyPostcond (void (*)(void*, GPattern*, ulong),
			 const void* closure,
			 GAddOrRemove = GADD) const;
    void destroyPostcond (void (*)(void*, const GPattern*),
			  const void* closure,
			  GAddOrRemove = GADD) const;
    // friends
    friend void Call_call_modify_postcond (GPattern* This,
					   const GShPattern*, ulong);

  private:
    GShPattern* _sh_pattern;


  protected:
    GPatternCondFlags _flags;
    GAssoc _assoc;

    int _addCallback (const char* type, GCBProc proc,
		      const void* closure) const;
    int _remCallback (const char* type, GCBProc proc,
		      const void* closure) const;

    GBool _call_modify_precond (ulong& modify_mask, GPatternValues*);
    void _call_modify_postcond (ulong modify_mask);
    void _call_destroy_postcond ();
};

#endif /* _Pattern_h */
