/* $Id: Pen.h,v 1.1.1.1 1992/03/18 09:56:46 uep Exp $ */
/*****************************************************************************
MODULE:  Tools/GC
FILE:    Pen.h
AUTHOR:  Jacques Davy
DATE:    November 1990

DESCRIPTION:
    This file describes the Pen object.

    The Pen object provides a regular object interface of a shared object
    ShPen (see ShPen.h). It also allows to set some pre/post conditions.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _Pen_h
#define _Pen_h

#include <Go/Tools/ShPen.h>
#include <Go/Tools/Assoc.h>
#include <Go/Tools/Callback.h>


/*
 * Condition flags
 */
struct GPenCondFlags
{
    GBool _has_modify_precond : 1;
    GBool _has_modify_postcond : 1;
    GBool _has_destroy_postcond : 1;

    GPenCondFlags() {
	_has_modify_precond  = GFALSE;
	_has_modify_postcond  = GFALSE;
	_has_destroy_postcond  = GFALSE;
    }
};

/*
 * GPen class definition
 */
class GPen
{
  private:
    GShPen* _sh_pen;


  protected:
    GPenCondFlags _flags;
    GAssoc  _assoc;

    int _addCallback (const char* type, GCBProc proc, const void* closure) const;
    int _remCallback (const char* type, GCBProc proc, const void* closure) const;

    GBool _call_modify_precond (ulong& modify_mask, GPenValues* new_values);
    void _call_modify_postcond (ulong modify_mask);
    void _call_destroy_postcond ();


  public:
    GPen ();
    GPen (const GPen* pen);
    GPen (const GShPen* sh_pen);
    GPen (ulong mask, const GPenValues* values, const char* name = NULL);
    GPen (const char* name);
    ~GPen ();

    // Getting values
    inline const GShPen* shPen() const { return _sh_pen; }
    inline       ulong mask() const { return _sh_pen->mask(); }
    inline const GPenValues* values() const { return _sh_pen->values(); }

    inline const char* name() const { return _sh_pen->name(); }
    inline GBool isNamed() const { return _sh_pen->isNamed(); }

    inline ulong function () const { return _sh_pen->function(); }
    inline ulong planeMask () const { return _sh_pen->planeMask(); }
    inline const class GShColor* color () const { return _sh_pen->color(); }
    inline const class GShColor* altColor () const { return _sh_pen->altColor(); }

    inline GNumber lineWidth () const { return _sh_pen->lineWidth(); }
    inline ushort lineStyle () const { return _sh_pen->lineStyle(); }
    inline ushort capStyle () const { return _sh_pen->capStyle(); }
    inline ushort joinStyle () const { return _sh_pen->joinStyle(); }
    inline GNumber miterLimit () const { return _sh_pen->miterLimit(); }
    inline const class GShDash* dash () const { return _sh_pen->dash(); }

    inline ushort fillStyle () const { return _sh_pen->fillStyle(); }
    inline const class GShPattern* tile () const { return _sh_pen->tile(); }
    inline const class GShPattern* stipple () const { return _sh_pen->stipple(); }
    inline GNumber xtsOrigin () const { return _sh_pen->xtsOrigin(); }
    inline GNumber ytsOrigin () const { return _sh_pen->ytsOrigin(); }

    // Modifying methods
    void modify (ulong mask, const GPenValues* values);
    void modify (const char* name);
    void modify (const GShPen* sh_pen);

    void function (ulong function);
    void planeMask (ulong plane_mask);
    void color (const class GColor* color);
    void color (const class GShColor* sh_color);
    void color (const char* color_name);
    void altColor (const class GColor* alt_color);
    void altColor (const class GShColor* sh_alt_color);
    void altColor (const char* alt_color_name);

    void lineWidth (GNumber line_width);
    void lineStyle (ushort line_style);
    void capStyle (ushort cap_style);
    void joinStyle (ushort join_style);
    void miterLimit (GNumber miter_limit);
    void dash (const class GDash* dash);
    void dash (const class GShDash* sh_dash);
    void dash (const char* dash_name);

    void fillStyle (ushort fill_style);
    void tile (const class GPattern* tile);
    void tile (const class GShPattern* sh_tile);
    void tile (const char* tile_name);
    void stipple (const class GPattern* stipple);
    void stipple (const class GShPattern* sh_stipple);
    void stipple (const char* stipple_name);
    void tsOrigin (GNumber x_origin, GNumber y_origin);

    // Pre and Post Conditions
    void modifyPrecond (GBool (*)(void*, const GPen*, ulong&, GPenValues*),
			const void* closure,
			GAddOrRemove = GADD) const;
    void modifyPostcond (void (*)(void*, GPen*, ulong), const void* closure,
			 GAddOrRemove = GADD) const;
    void destroyPostcond (void (*)(void*, const GPen*), const void* closure,
			  GAddOrRemove = GADD) const;
    // friends
    friend void Call_call_modify_postcond (GPen* This, const GShPen*, ulong); 
};


#endif /* _Pen_h */
