/* $Id: Point.h,v 1.2 1992/09/09 07:28:02 smadja Exp $ */
/*****************************************************************************
MODULE:  Tools
FILE:    Point.h
AUTHOR:  Olivier Jojic
DATE:    Jan 90

DESCRIPTION:

The Point class  with classic functions.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/
#ifndef _Point_h
#define _Point_h

#include <Go/Tools/Vector.h>

struct GPoint : public GVector
{
    GPoint();
    GPoint(GNumber x, GNumber y);
    GPoint(const GVector&);

    friend GNumber GSquareDistance(const GPoint&, const GPoint&);
    friend GNumber GSquareDistance(const GPoint&,
				   const GPoint& lineP1, const GPoint& lineP2);

    friend GNumber GDistance(const GPoint&, const GPoint&);
    friend GNumber GDistance(const GPoint&,
			     const GPoint& lineP1, const GPoint& lineP2);

    friend GPoint operator + (const GPoint&, const GPoint&);
    friend GPoint operator + (const GPoint&, const GVector&);
};



/*
 * inline methods:
 */

inline GPoint::GPoint ()
{
}

inline GPoint::GPoint (GNumber x, GNumber y) : GVector(x, y)
{
}

inline GPoint::GPoint (const GVector& v) : GVector(v)
{
}

inline GNumber GSquareDistance (const GPoint& a, const GPoint& b)
{
    return (b - a).squareSize();
}

inline GNumber GDistance (const GPoint& a, const GPoint& b)
{
    return sqrt(GSquareDistance(a, b));
}

inline GNumber GDistance (const GPoint& a, const GPoint& b1, const GPoint& b2)
{
    return sqrt(GSquareDistance(a, b1, b2));
}

inline GPoint operator + (const GPoint& a, const GPoint& b)
{
    return (const GVector&)a + (const GVector&)b;
}

inline GPoint operator + (const GPoint& a, const GVector& b)
{
    return (const GVector&)a + (const GVector&)b;
}

#endif /* _Point_h */
