/* $Id: Queue.h,v 1.3 1992/12/16 16:02:08 jojic Exp $ */
/*****************************************************************************
MODULE:  Tools
FILE:    Queue.h
AUTHOR:  Oliver Jojic
DATE:    October 1990

DESCRIPTION:
    
COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _Queue_h
#define _Queue_h

#include <Go/Tools/Error.h>
#include <Go/machdef.h>
#include <Go/Tools/Alloc.h>

class GSet;


class GQueue
{
  public:
    GQueue (int initial_size = 32);
    ~GQueue ();

    GBool empty () const;
    int length () const;
    void* getfirst ();
    void* getlast ();
    void putfirst (void*);
    void putlast (void*);

    void putfirst (GQueue& q) { copyfirst(q); q.last = q.first; }
    void putlast (GQueue& q) { copylast(q); q.last = q.first; }

    void** arrayFromFirst (int& count) const;
    void clear () { last = first; }

    //methods below allows to get q elements without popping them.
    void initRef (int& ref) const { ref = first; }
    void prevRef (int& ref) const { --ref; }
    void nextRef (int& ref) const { ref++; }
    GBool getRef (int& ref, void*& value) const;
    GBool remRef (int& ref);
    void copyfirst (const GQueue&);
    void copylast (const GQueue&);
    void copyTo (GSet&) const;

  private:
    int size;
    void** array;
    int first;
    int last;

    void incr_size_for_first ();
    void incr_size_for_last ();

};



/*
 * inline methods:
 */

inline GQueue::~GQueue ()
{
    GFREE (array);
}

inline GBool GQueue::empty () const
{
    return first >= last;
}

inline int GQueue::length () const
{
    return last - first;
}

inline void* GQueue::getfirst ()
{
    if (empty())
	GError ("GQueue::getfirst() is empty !");

    return array[first++];
}

inline void* GQueue::getlast ()
{
    if (empty())
	GError ("GQueue::getlast() is empty !");

    return array[--last];
}

inline void GQueue::putfirst (void* value)
{
    if (first == 0)
	incr_size_for_first();
    array[--first] = value;
}

inline void GQueue::putlast (void* value)
{
    if (last == size)
	incr_size_for_last();
    array[last++] = value;
}

inline GQueue::GQueue (int initial_size)
{
    size = initial_size;
    array = (void**) GMALLOC (size * sizeof (void*));
    first = last = initial_size / 2;
}

inline void** GQueue::arrayFromFirst (int& count) const
{
    count = last - first;
    return array + first;
}


#endif /* _Queue_h */
