/* $Id: Region.h,v 1.2 1992/05/26 16:34:16 benati Exp $ */
/*****************************************************************************
MODULE:  Tools
FILE:    Region.h
AUTHOR:  Oliver Jojic
DATE:    July 90

DESCRIPTION:
    

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _Region_h
#define _Region_h

#include <Go/Tools/Box.h>
#include <Go/Tools/Error.h>


extern int GPositionToPixel(GNumber pos_in_mm, GNumber pixel_size);

class GQueue;

struct GIntBox
{
    short x, y;
    ushort width, height;
};


class GRegion
{
  public:
    GRegion();
    GRegion(int); // number of allocated boxes;
    GRegion(const GBox&);
    GRegion(const GRegion&);
    ~GRegion();

    void operator = (const GRegion&);
    void operator = (const GBox&);

    friend GBool operator == (const GRegion&, const GRegion&);
    friend GBool operator != (const GRegion& a, const GRegion& b);

    friend GBool operator >= (const GRegion&, const GRegion&);
    friend GBool operator >  (const GRegion& a, const GRegion& b)
	{ return a >= b && a != b; }
    friend GBool operator <= (const GRegion& a, const GRegion& b)
	{ return b >= a; }
    friend GBool operator <  (const GRegion& a, const GRegion& b)
	{ return b >= a && a != b; }

    friend GBool operator && (const GRegion&, const GRegion&);
    friend GBool operator && (const GRegion&, const GBox&);
    friend GBool operator && (const GBox& b, const GRegion& r)
	{ return r && b; }

    void operator += (const GBox&);
    void operator -= (const GBox& b) { GSub(*this, b, *this); }
    void operator += (const GRegion& r) { GAdd(*this, r, *this); }
    void operator -= (const GRegion& r) { GSub(*this, r, *this); }

    friend void GAdd (const GRegion&, const GRegion&, GRegion& result);
    friend void GAdd (const GRegion&, const GBox&, GRegion& result);
    friend void GAdd (const GBox& b, const GRegion& r, GRegion& result)
	{ GAdd(r, b, result); }
    friend void GAdd (const GRegion&, const GRegion&, GRegion&,
		      GNumber mindist);

    friend void GSub (const GRegion&, const GRegion&, GRegion& result);
    friend void GSub (const GRegion&, const GBox&, GRegion& result);
    friend void GSub (const GBox&, const GRegion&, GRegion& result);

    friend void GIntersect (const GRegion&, const GRegion&, GRegion& result);
    friend void GIntersect (const GRegion&, const GBox&, GRegion& result);
    friend void GIntersect (const GBox& b, const GRegion& r, GRegion& result)
	{ GIntersect(r, b, result); }

    const GBox& bigbox() const { return big; }
    GBool exist(int i) const { return i >= 0 && i < count; }
    const GBox& box(int i) const { return boxes[i]; }
    int numboxes() const { return count; }
    GBool empty() const { return count == 0; }
    GBool contains(const GPoint&, GNumber maxdist = 0) const;

    void setEmpty();

    GIntBox* pixelBoxes
	(GNumber pix_width, GNumber pix_height, int& num_boxes) const;
    GNumber squareArea() const;
    GBool squareAreaBiggerThan (GNumber) const;

  private:
    GBox big;
    ushort size;
    ushort count;
    GBox* boxes;
    
    void initialize(int);
    void set_from_box_queue(const GQueue&);
    void add_box_at_end(const GBox&);
    void recompute_bigbox();
    void incr_size(int = 0);
    friend void common_GAdd (const GBox*, const GBox*,
			     const GBox*, const GBox*, GRegion&);
    friend void common_GSub (const GBox*, const GBox*,
			     const GBox*, const GBox*, GRegion&);
};

const GRegion& GEmptyRegion();
const GRegion& GBiggestRegion();

#endif /* _Region_h */
