/* $Id: ShArray.h,v 1.3 1992/09/25 11:51:03 smadja Exp $ */
/*****************************************************************************
MODULE:  TOOLS SH ARRAY
FILE:    ShArray.h
AUTHOR:  Philippe Smadja
DATE:    Dec 1990

DESCRIPTION:
  Used to share an Array. the match function uses memcmp.
  The AhArray object inherits of SharedOBject class.
  
COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
*****************************************************************************/
#ifndef _ShArray_h
#define _ShArray_h

#include <Go/Tools/SharedObj.h>
#include <Go/Tools/Array.h>

#define GSHARRAY_MASK (ulong)-1

struct GShArrayValues {
    void* data;           /** it is the address of a table
			    of nelems. the size of an element
			    is elemsize.
			    **/
    int   elemsize;
    int   nelems;
} ;

class GShArray : public GSharedObject {

  public:

    static const char* classname();
    virtual const char* getClassname() const;
    static int save(const GShArray*, GoStream*);
    static int restore(GShArray**, GoStream*);
    static int readParams(GShArray*, GoStream*);

    inline  void* data() { return _array->data();};
    inline  int   nelems() { return _array->nelems();};
    inline  int   elemsize() { return _array->elemsize(); } ;
    
    friend  GShArray* GCreateShArray (const GShArrayValues* values);
    friend  GShArray* GCreateShArray (void* data, int nelems = 1,
				      int elemsize = sizeof(void*));
    friend  void     GAddElem(GShArray*&, void* elem, int nelems_to_add = 1);
    friend  void     GRemElem(GShArray*&, int index, int nelems_to_free = 1);

  protected:
    
    GList*& _head();
    int    _valueSize ();
    virtual GBool  _match (ulong mask, const void * values) const ;
    virtual void   _makeNewValues (ulong mask, const void * values,
				   ulong& newMask, void * newValues) const ;
    virtual ulong  _update (ulong mask, const void * values);

    friend GShArray* _newGShArray (GFunctionList* funcList,
				   ulong mask,
				   const void * values,
				   const char* name = NULL);

    GShArray (GFunctionList* funcList, ulong mask,
	      const GShArrayValues* values, const char* name = NULL);
    virtual ~GShArray ();

  private:

    static GList* _headOfList;
    static GFunctionList _funcList;

    GArray* _array;
};

#endif /* _ShArray_h */
