/* $Id: ShFont.h,v 1.4 1992/09/25 11:51:48 smadja Exp $ */
/*****************************************************************************
MODULE:  Tools
FILE:    ShFont.h
AUTHOR:  Jacques Davy
DATE:    15 June 1990

DESCRIPTION:
    The ShFont object allows to get information about a given font
    in a given size.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  license from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _ShFont_h
#define _ShFont_h

#include <Go/machdef.h>
#include <Go/Tools/SharedObj.h>
#include <Go/Tools/Number.h>

#define GFONTNAME_SEPARATOR "-"
#define GFONTNAME_UNDEFINED_FIELD "*"

/* Mask definitions for GSharedObject operations */
#define GFONT_REGISTRY      (1L<<0)
#define GFONT_FAMILY        (1L<<1)
#define GFONT_WEIGHT        (1L<<2)
#define GFONT_SLANT         (1L<<3)
#define GFONT_WIDTH         (1L<<4)
#define GFONT_ADD_STYLE     (1L<<5)
#define GFONT_SPACING       (1L<<6)
#define GFONT_CHARSET       (1L<<7)
#define GFONT_CHARCODE      (1L<<8)

/* Value structure for GSharedObject operations */
struct GFontValues {
   char* registry;
   char* family;
   char* weight;
   char* slant;
   char* width;
   char* add_style;
   char* spacing;
   char* charset_registry;
   char* charset_encoding;
};

/* GShFont class definition */
class GShFont : public GSharedObject
{
    friend class GFont;

  public:

    friend void GModify (GShFont*& sho, ulong mask,
			 const GFontValues* values);
    friend void GModify (const char* name, ulong mask,
			 const GFontValues* values);
    friend void GRename (GShFont*& sho, const char* name);

    static int save(const GShFont*, GoStream*);
    static int restore(GShFont**, GoStream*);
    static const char* classname();
    static int readParams(GShFont*, GoStream*);

    virtual const char* getClassname() const;

    inline GShFont* dup() const;
    inline GBool match (const GShFont* sho);
    inline void updatePostcond (void (*)(void*, GShFont*, ulong),
				const void* closure, GAddOrRemove = GADD) const;
    inline void incrRefPostcond (void (*)(void*, GShFont*),
				 const void* closure, GAddOrRemove = GADD) const;
    inline void decrRefPostcond (void (*)(void*, GShFont*),
				 const void* closure, GAddOrRemove = GADD) const;
    inline void destroyPostcond (void (*)(void*, const GShFont*),
				 const void* closure, GAddOrRemove = GADD) const;

    friend GShFont* GCreateShFont (ulong mask, const GFontValues* values,
				   const char* name = NULL);
    friend GShFont* GCreateShFont (const char* font_name);
    friend GShFont* GCreateShFont ();

    // Getting values;
    inline const GFontValues* values () const { return &_values; }
    char* fullName () const;
    inline const char* registry () const {return _values.registry; }
    inline const char* family () const { return _values.family; }
    inline const char* weight () const { return _values.weight; }
    inline const char* slant () const { return _values.slant; }
    inline const char* width () const { return _values.width; }
    inline const char* addStyle () const { return _values.add_style; }
    inline const char* spacing () const { return _values.spacing; }
    inline const char* charset () const { return _values.charset_registry; }
    inline const char* charcode () const { return _values.charset_encoding; }

    // Getting string metrics (see GoInterf.h);

  protected:

    inline GList*& _head () { return _headOfList; }
    inline int _valueSize () { return (sizeof(GFontValues)); }
    virtual GBool _match (ulong mask, const void* values) const;
    virtual void _makeNewValues (ulong mask, const void* values,
				 ulong& newMask, void* newValues) const;
    virtual ulong _update (ulong mask, const void* values);
    friend GShFont* _newGShFont (GFunctionList* funcList, ulong mask,
				 const void* values,
				 const char* name = NULL);

    GShFont (GFunctionList* funcList, ulong mask, const GFontValues* values,
	     const char* name = NULL);
    virtual ~GShFont ();


  private:

    GFontValues _values;
    static GList* _headOfList;
    static GFunctionList _funcList;
};

/*
 * Inline Methods
 */
inline void GModify (GShFont*& sho, ulong mask, const GFontValues* values)
{
    sho->modify((GSharedObject*&)sho, mask, (const void*)values);
}

/**/
inline void GRename (GShFont*& sho, const char* name)
{
   sho->rename((GSharedObject*&)sho, name);
}

/**/
inline GShFont* GShFont::dup() const
{
    return (GShFont*)((GSharedObject*)this)->dup();
}

/**/
inline GBool GShFont::match (const GShFont* sho)
{
    return _match (sho->mask(), sho->values());
}

/**/
inline void GShFont::updatePostcond (void cb(void*, GShFont*, ulong),
				    const void* closure, GAddOrRemove add) const
{
    GSharedObject::updatePostcond (
   (void(*)(void*, GSharedObject*, ulong))cb, closure, add);
}

/**/
inline void GShFont::incrRefPostcond (void cb(void*, GShFont*),
				     const void* closure, GAddOrRemove add) const
{
    GSharedObject::incrRefPostcond (
		          (void(*)(void*, GSharedObject*))cb, closure, add);
}

/**/
inline void GShFont::decrRefPostcond (void cb(void*, GShFont*),
				     const void* closure, GAddOrRemove add) const
{
    GSharedObject::decrRefPostcond (
                         (void(*)(void*, GSharedObject*))cb, closure, add);
}

/**/
inline void GShFont::destroyPostcond (void cb(void*, const GShFont*),
				     const void* closure, GAddOrRemove add) const
{
    GSharedObject::destroyPostcond (
		          (void(*)(void*, const GSharedObject*))cb, closure, add);
}

#endif /* _ShFont_h */
