/* $Id: ShTextBrush.h,v 1.4 1992/09/25 11:52:42 smadja Exp $ */
/*****************************************************************************
MODULE:  Tools/GC
FILE:    ShTextBrush.h
AUTHOR:  Jacques Davy
DATE:    October 1990

DESCRIPTION:
    This file describes the ShTextBrush shared object.
    The ShTextBrush object is used by the GoInterf objects when drawing strings.
    The TextBrushValues structure may be used to set ShTextBrush parameters.
    Unset parameters have default values (se below).

    ShTextBrush objects are shared objects (SharedObject subclass). See
    also TextBrush objects.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  license from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _ShTextBrush_h
#define _ShTextBrush_h

#include <Go/machdef.h>
#include <Go/Tools/SharedObj.h>
#include <Go/Tools/Number.h>

/* Mask definitions for GSharedObject operations */
#include <Go/Tools/GCdef.h>

/* Value Structure for GSharedObject operations (with defaults) */
struct GTextBrushValues {
    ulong function;             // logical operation (Gcopy)
    ulong plane_mask;           // plane mask        (GallPlanes)
    class GShColor*  color;      // main color        (black)
    class GShColor*  alt_color;  // alternate color   (none)

    /* fill attributes */
    struct {
       unsigned style : 2;      // fill style (GfillSolid)
    } fill;
    class GShPattern* tile;      // tile pattern for tiling (none)
    class GShPattern* stipple;   // stipple 1 plane pattern for stippling (none)
    GNumber   ts_x_origin;       // x offset for tile/stipple operations (0.0)
    GNumber   ts_y_origin;       // y offset for tile/stipple operations (0.0)

    /* text attributes */
    class GShFont* font;         // text font
};

#define GMODIFY_TEXTBRUSHVALUE(text_brush, maskBit, field, newValue)\
    {\
        GTextBrushValues values;\
        values.field = newValue;\
        GModify(text_brush, maskBit, &values);\
    }

#define GUPDATE_TEXTBRUSHVALUE(name, maskBit, field, newValue)\
    {\
        GTextBrushValues values;\
	values.field = newValue;\
        GModify (name, maskBit, &values);\
    }

/* GShTextBrush class definition */
class GShTextBrush : public GSharedObject
{
    friend class GTextBrush;

  public:

    friend GShTextBrush* GCreateShTextBrush ();
    friend GShTextBrush* GCreateShTextBrush (ulong mask,
					     const GTextBrushValues* values,
					     const char* name = NULL);
    friend GShTextBrush* GCreateShTextBrush (const char* name);

    static int save(const GShTextBrush*, GoStream*);
    static int restore(GShTextBrush**, GoStream*);
    static const char* classname();
    static int readParams(GShTextBrush*, GoStream*);

    virtual const char* getClassname() const;

    inline GShTextBrush* dup() const;
    inline GBool match (const GShTextBrush* text_brush) const;
    inline void updatePostcond (void (*)(void*, GShTextBrush*, ulong),
				const void* closure, GAddOrRemove = GADD) const;
    inline void destroyPostcond (void (*)(void*, const GShTextBrush*),
				 const void* closure, GAddOrRemove = GADD) const;

    // Getting values;
    inline const GTextBrushValues* values() const { return &_values; }

    inline ulong function () const { return _values.function; }
    inline ulong planeMask () const { return _values.plane_mask; }
    inline const class GShColor* color () const { return _values.color; }
    inline const class GShColor* altColor () const { return _values.alt_color; }

    inline ushort fillStyle () const { return _values.fill.style; }
    inline const class GShPattern* tile () const { return _values.tile; }
    inline const class GShPattern* stipple () const { return _values.stipple; }
    inline GNumber xtsOrigin () const { return _values.ts_x_origin; }
    inline GNumber ytsOrigin () const { return _values.ts_y_origin; }

    inline const class GShFont* font() const { return _values.font; }

    // Modifying methods;
    friend void GModify (GShTextBrush*& text_brush, ulong mask,
			 const GTextBrushValues* values);
    friend void GRename (GShTextBrush*& text_brush, const char* name);

    friend void GSetFunction (GShTextBrush*& text_brush, ulong function);
    friend void GSetPlaneMask (GShTextBrush*& text_brush, ulong plane_mask);
    friend void GSetColor (GShTextBrush*& text_brush,
			   const class GShColor* color);
    friend void GSetAltColor (GShTextBrush*& text_brush,
			      const class GShColor* alt_color);

    friend void GSetFillStyle (GShTextBrush*& text_brush, ushort fill_style);
    friend void GSetTile (GShTextBrush*& text_brush,
			  const class GShPattern* tile);
    friend void GSetStipple (GShTextBrush*& text_brush,
			     const class GShPattern* stipple);
    friend void GSetTSOrigin (GShTextBrush*& text_brush, GNumber x_origin,
			      GNumber y_origin);

    friend void GSetFont (GShTextBrush*& text_brush, const class GShFont* font);

    // updating named GShTextBrush;

    friend void GModify (const char* name, ulong mask,
			 const GTextBrushValues* values);

    friend void GTextBrushFunction (const char* name, ulong function);
    friend void GTextBrushPlaneMask (const char* name, ulong plane_mask);
    friend void GTextBrushColor (const char* name,
				 const class GShColor* color);
    friend void GTextBrushAltColor (const char* name,
				    const class GShColor* alt_color);

    friend void GTextBrushFillStyle (const char* name, ushort fill_style);
    friend void GTextBrushTile (const char* name,
				const class GShPattern* tile);
    friend void GTextBrushStipple (const char* name,
				   const class GShPattern* stipple);
    friend void GTextBrushTsOrigin (const char* name, GNumber x_origin,
				    GNumber y_origin);

    friend void GTextBrushFont (const char* name,
				const class GShFont* font);
    // friends for callbacks;
    friend void   ColorIsUpdated(GShTextBrush* This, const GShColor* color,
				 ulong mask);
    friend void   FontIsUpdated(GShTextBrush* This, const GShFont* font,
				ulong mask);
    friend void   PatternIsUpdated(GShTextBrush* This, const GShPattern* pattern,
				   ulong mask);
    
  protected:

    inline GList*& _head () { return _headOfList; }
    inline int  _valueSize () { return (sizeof(GTextBrushValues)); }
    virtual GBool _match (ulong mask, const void* values) const;
    virtual void _makeNewValues (ulong mask, const void* values,
				 ulong& newMask, void* newValues) const;
    virtual ulong _update (ulong mask, const void* values);
    friend GShTextBrush* _newGShTextBrush (GFunctionList* funcList, ulong mask,
					   const void* values,
					   const char* name = NULL);

    GShTextBrush (GFunctionList* funcList, ulong mask,
		  const GTextBrushValues* values, const char* name = NULL);
    virtual ~GShTextBrush ();


  private:

    GTextBrushValues _values;
    static GList* _headOfList;
    static GFunctionList _funcList;
};

/*
 * Inline Methods
 */
inline void GModify (GShTextBrush*& text_brush, ulong mask, const GTextBrushValues* values)
    { text_brush->modify((GSharedObject*&)text_brush, mask, (const void*)values); }

inline void GRename (GShTextBrush*& text_brush, const char* name)
    { text_brush->rename ((GSharedObject*&)text_brush, name); }

inline void GSetFunction (GShTextBrush*& text_brush, ulong function)
    { GMODIFY_TEXTBRUSHVALUE(text_brush, GFUNCTION, function, function); }

inline void GSetPlaneMask (GShTextBrush*& text_brush, ulong plane_mask)
    { GMODIFY_TEXTBRUSHVALUE(text_brush, GPLANE_MASK, plane_mask, plane_mask); }

inline void GSetColor (GShTextBrush*& text_brush, const class GShColor* color)
    { GMODIFY_TEXTBRUSHVALUE(text_brush, GCOLOR, color, (class GShColor*)color); }

inline void GSetAltColor (GShTextBrush*& text_brush, const class GShColor* alt_color)
    { GMODIFY_TEXTBRUSHVALUE(text_brush, GALT_COLOR, alt_color, (class GShColor*)alt_color); }

inline void GSetFillStyle (GShTextBrush*& text_brush, ushort fill_style)
    { GMODIFY_TEXTBRUSHVALUE(text_brush, GFILL_STYLE, fill.style, fill_style); }

inline void GSetTile (GShTextBrush*& text_brush, const class GShPattern* tile)
    { GMODIFY_TEXTBRUSHVALUE(text_brush, GTILE, tile, (class GShPattern*)tile); }

inline void GSetStipple (GShTextBrush*& text_brush, const class GShPattern* stipple)
    { GMODIFY_TEXTBRUSHVALUE(text_brush, GSTIPPLE, stipple, (class GShPattern*)stipple); }

inline void GSetTSOrigin (GShTextBrush*& text_brush, GNumber x_origin, GNumber y_origin)
{
    GTextBrushValues values;
    values.ts_x_origin = x_origin;
    values.ts_y_origin = y_origin;
    GModify (text_brush, GTILE_STIP_XORIGIN | GTILE_STIP_YORIGIN, &values);    
}

inline void GSetFont (GShTextBrush*& text_brush, const class GShFont* font)
    { GMODIFY_TEXTBRUSHVALUE(text_brush, GFONT, font, (class GShFont*)font); }

/**/
inline void GTextBrushFunction (const char* name, ulong function)
    { GUPDATE_TEXTBRUSHVALUE (name, GFUNCTION, function, function); }

inline void GTextBrushPlaneMask (const char* name, ulong plane_mask)
    { GUPDATE_TEXTBRUSHVALUE (name, GPLANE_MASK, plane_mask, plane_mask); }

inline void GTextBrushColor (const char* name, const class GShColor* color)
    { GUPDATE_TEXTBRUSHVALUE (name, GCOLOR, color, (class GShColor*)color); }

inline void GTextBrushAltColor (const char* name, const class GShColor* alt_color)
    { GUPDATE_TEXTBRUSHVALUE (name, GALT_COLOR, alt_color, (class GShColor*)alt_color); }

inline void GTextBrushFillStyle (const char* name, ushort fill_style)
    { GUPDATE_TEXTBRUSHVALUE (name, GFILL_STYLE, fill.style, fill_style); }


inline void GTextBrushTile (const char* name, const class GShPattern* tile)
    { GUPDATE_TEXTBRUSHVALUE (name, GTILE, tile, (class GShPattern*)tile); }

inline void GTextBrushStipple (const char* name, const class GShPattern* stipple)
    { GUPDATE_TEXTBRUSHVALUE (name, GSTIPPLE, stipple, (class GShPattern*)stipple); }

inline void GTextBrushTsOrigin (const char* name, GNumber x_origin, GNumber y_origin)
{
    GTextBrushValues values;
    values.ts_x_origin = x_origin;
    values.ts_y_origin = y_origin;
    GCreateShTextBrush (GTILE_STIP_XORIGIN | GTILE_STIP_YORIGIN, &values, name);    
}

inline void GTextBrushFont (const char* name, const class GShFont* font)
    { GUPDATE_TEXTBRUSHVALUE (name, GFONT, font, (class GShFont*)font); }

/**/
inline GShTextBrush* GShTextBrush::dup() const
    { return (GShTextBrush*)((GSharedObject*)this)->dup(); }

/**/
inline GBool GShTextBrush::match (const GShTextBrush* text_brush) const
    { return ((GShTextBrush*)this)->_match (text_brush->mask(), text_brush->values()); }

/**/
inline void GShTextBrush::updatePostcond (void cb(void*, GShTextBrush*, ulong),
				 const void* closure, GAddOrRemove add) const
{
    GSharedObject::updatePostcond (
                  (void(*)(void*, GSharedObject*, ulong))cb, closure, add);
}

/**/
inline void GShTextBrush::destroyPostcond (void cb(void*, const GShTextBrush*),
				  const void* closure, GAddOrRemove add) const
{
    GSharedObject::destroyPostcond (
		          (void(*)(void*, const GSharedObject*))cb, closure, add);
}

#undef GMODIFY_TEXTBRUSHVALUE
#undef GUPDATE_TEXTBRUSHVALUE

#endif /* _ShTextBrush_h */
