/* $Id: StringAssoc.h,v 1.2 1992/09/30 13:57:42 kuta Exp $ */
/*****************************************************************************
MODULE:  Tools
FILE:    StringAssoc.h
AUTHOR:  Olivier Jojic
DATE:    10 June 1989

DESCRIPTION:
    The "StringAssoc" class allows you to associate one or more values to some keys.
    "key" is a name (i.e. strcmp is used to compare key). If you want to use
    integer-like keys you must use the "Assoc" class.

    Retriving of values for a given key is done using a hash-coding.
    The size of the hash-table is settable by the programmer.
    By default, while the number of associations if small, the hash-table has
    only one entry. And when the number of associations increases, the hash-table
    behave automatically 256 entries.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _StringAssoc_h
#define _StringAssoc_h

#include <Go/machdef.h>



class GStringAssoc
{
  public:
    GStringAssoc();
    ~GStringAssoc();

    void  setArraySize(int); //must be a power of 2 <= 256

    void  set (const char* key, const void* value);
    void  set (const char* key, int size, const void* addr);
    void  add (const char* key, const void* elem);
    void  add (const char* key, int size, const void* addr);
    void* get (const char* key) const;
    const void* get (const char* key, int& numElem) const;
    void  rem (const char* key);
    void  rem (const char* key, const void* elem);
    void  rem (const char* key, int size, const void* addr);

    void reset();
    char** getKeys(int& count) const;

  private:
    ushort              arraySize;
    class StringLinearAssoc** array;

    int  hash(const char* key) const;
    int  keyToIndex(const char* key) const;
    void maybeSwitchToHash();
};


#endif /* _StringAssoc_h */
