/* $Id: TreeNode.h,v 1.5 1992/12/11 08:35:29 jojic Exp $ */
/*****************************************************************************
MODULE:  BULL Graphic Objects
FILE:    TreeNode.h
AUTHOR:  Aline Jacomet
DATE:    January 11th 1991

DESCRIPTION:
         Definition of the tree node object class.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _GoTreeNode_h
#define _GoTreeNode_h

#include <Go/Go.h>
#include <Go/Node.h>
#include <Go/mTreeNode.h>
#include <Go/Link.h>


/*
 * GoTreeNode definition
 */
class GoTreeNode : public GoNode {
    friend class GoTree;
    friend class GoTreeNodeMetaclass;
    friend class GoGraphStruct;
    
  public:
    /*
     * get ...;
     */
    inline GoTreeNode* nextNode() const {
	return next_node;
    }
    inline GoTreeNode* previousNode() const {
	return prev_node;
    }
    inline GoTreeNode* parentNode() const {
	return parent_node;
    }
    inline GoTreeNode* childNode() const {
	return child_node;
    }
    inline GoLink* parentLink() const {
	return parent_link;
    }
    inline GBool isOpened() const {
	return _open;
    }

    /*
     * Pre and Post Conditions ...;
     */
    void attachNodePrecond(GBool (*)(void*, GoTreeNode*, GoTreeNode*,
				     GoMetaclass*, GoTreeNode*, GoTree*),
			   void* closure, GAddOrRemove = GADD);
    void attachNodePostcond(void (*)(void*, GoTreeNode*, GoTreeNode*,
				     GoMetaclass*, GoTreeNode*, GoTree*),
			    void* closure, GAddOrRemove = GADD);
    void shiftNodePrecond(GBool (*)(void*, GoTreeNode*, GoTreeNode*),
			  void* closure, GAddOrRemove = GADD);
    void shiftNodePostcond(void (*)(void*, GoTreeNode*, GoTreeNode*),
			   void* closure, GAddOrRemove = GADD);
    
    /*
     * others ...;
     */
    static GoTreeNode*          create();
    static GoTreeNodeMetaclass* metaclass();

    void openNode();
    void closeNode();
    GBool shiftNode(GoTreeNode* next = NULL);
    GBool attachNode(GoTreeNode*, GoMetaclass* linkclass = NULL,
		     GoTreeNode* next = NULL, GoTree* graph = NULL);

    /*
     * save, restore;
     */
    static int save(const GoTreeNode*, GoStream*);
    static int restore(GoTreeNode**, GoStream*);
    static int readParams(GoTreeNode*, GoStream*);

  protected:
    GoTreeNodeCondFlags _flags;

    virtual void m_dup(const Go*, Go*, Go*, GBool, GBool);
    GoTreeNode();
    virtual ~GoTreeNode();


    GBool _call_attach_node_precond(GoTreeNode*, GoMetaclass*, GoTreeNode*,
				    GoTree*) const;
    void _call_attach_node_postcond(GoTreeNode*, GoMetaclass*, GoTreeNode*,
				    GoTree*);
    GBool _call_shift_node_precond(GoTreeNode*) const;
    void _call_shift_node_postcond(GoTreeNode*);
    GBool _call_class_attach_node_precond(GoTreeNode*, GoMetaclass*,
					  GoTreeNode*, GoTree*) const;
    void  _call_class_attach_node_postcond(GoTreeNode*, GoMetaclass*,
					   GoTreeNode*, GoTree*);
    GBool _call_class_shift_node_precond(GoTreeNode*) const;
    void  _call_class_shift_node_postcond(GoTreeNode*);

  private:
    GoTreeNode* parent_node;
    GoTreeNode* child_node;
    GoTreeNode* next_node;
    GoTreeNode* prev_node;
    GoLink*     parent_link;
    GBool       _open;
    GBool       first_close;   // indicates if subtree is closed.
    GBool       _marked;       // only used by placement algorythm;
    GNumber     _modif;        // only used by placement algorythm;
    
    virtual GoMetaclass* m_metaclass() const;
    
    void  reparent_on_tree(GoTree*);
    void  open_tree();
    void  close_tree();
    GBool is_parent_correct(GoTreeNode*);
};

INLINE GoTreeNode* GoTreeNode::create() {
    return (GoTreeNode*) Go::Create(GoTreeNode::metaclass());
}


#endif /* _GoTreeNode_h */
