/* $Id: mGraph.h,v 1.1.1.1 1992/03/18 10:07:16 uep Exp $ */
/*****************************************************************************
MODULE:  BULL Graphic Objects
FILE:    mGraph.h
AUTHOR:  Aline Jacomet
DATE:    February 8th 1991

DESCRIPTION:
         Definition of the graph object metaclass.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _mGoGraph_h
#define _mGoGraph_h

#include <Go/mGraphStr.h>


/*
 * Condition Flags
 */
struct GoGraphCondFlags {
    GBool _has_add_node_precond : 1;
    GBool _has_add_node_postcond : 1;
    GBool _has_remove_node_precond : 1;
    GBool _has_remove_node_postcond : 1;
    GBool _has_add_link_precond : 1;
    GBool _has_add_link_postcond : 1;
    GBool _has_remove_link_precond : 1;
    GBool _has_remove_link_postcond : 1;
    GoGraphCondFlags() {
	_has_add_node_precond = GFALSE;
	_has_add_node_postcond = GFALSE;
	_has_remove_node_precond = GFALSE;
	_has_remove_node_postcond = GFALSE;
	_has_add_link_precond = GFALSE;
	_has_add_link_postcond = GFALSE;
	_has_remove_link_precond = GFALSE;
	_has_remove_link_postcond = GFALSE;
    }
};


class GoGraphMetaclass : public GoGraphStructMetaclass {
    friend class GoGraph;
    
  public:
    /*
     * Pre and Post Conditions ...;
     */
    void addNodePrecond(GBool (*)(void*, GoGraph*, GoGraphNode*, GBool*,
				  GoMetaclass*),
			void* closure, GAddOrRemove = GADD);
    void addNodePostcond(void (*)(void*, GoGraph*, GoGraphNode*, GBool*,
				  GoMetaclass*),
			 void* closure, GAddOrRemove = GADD);
    void removeNodePrecond(GBool (*)(void*, GoGraph*, GoGraphNode*),
			   void* closure, GAddOrRemove = GADD);
    void removeNodePostcond(void (*)(void*, GoGraph*, GoGraphNode*),
			    void* closure, GAddOrRemove = GADD);
    void addLinkPrecond(GBool (*)(void*, GoGraph*, GoGraphNode*, GoGraphNode*,
				  GoMetaclass*),
			void* closure, GAddOrRemove = GADD);
    void addLinkPostcond(void (*)(void*, GoGraph*, GoGraphNode*, GoGraphNode*,
				  GoMetaclass*),
			 void* closure, GAddOrRemove = GADD);
    void removeLinkPrecond(GBool (*)(void*, GoGraph*, GoLink*),
			   void* closure, GAddOrRemove = GADD);
    void removeLinkPostcond(void (*)(void*, GoGraph*, GoLink*),
			    void* closure, GAddOrRemove = GADD);
    
  protected:
    GoGraphCondFlags _flags;
    
    GoGraphMetaclass();
    virtual ~GoGraphMetaclass();
    
  private:
    virtual const char*      m_classname() const;
    virtual GoMetaclass*     m_superclass() const;
    virtual Go*              m_create() const;
    virtual const GActionRec* m_actions() const;
};


#endif /* _mGoGraph_h */

