/* $Id: mTextual.h,v 1.2 1992/08/06 11:50:34 uep Exp $ */
/***************************************************************************
MODULE:  GoTextual
FILE:    mTextual.h
AUTHOR:  Michel ANDRE
DATE:    March 1991

DESCRIPTION :

  GoTextual Metaclass Description


COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
***************************************************************************/
#ifndef _mGoTextual_h
#define _mGoTextual_h

#include <Go/mGoStdDS.h>
#include <Go/TextDefs.h>


/***********************
* condition flags
***********************/

struct GoTextualCondFlags {
    GBool _has_modify_value_precond : 1;
    GBool _has_modify_value_postcond : 1;
    GBool _has_current_pos_precond : 1;
    GBool _has_current_pos_postcond : 1;
    GBool _has_modify_attribute_precond : 1;
    GBool _has_modify_attribute_postcond : 1;
    GBool _has_selection_precond : 1;
    GBool _has_selection_postcond : 1;
};



/***************************
*  class GoTextualMetaclass
****************************/

class GoTextualMetaclass : public GoStdDSMetaclass
{
    friend class		GoTextual;
    friend class		GoSimpleText;
    friend class		GoText;

  public:
    /* Pre/Postconds on modify value */
    void	modifyValuePrecond (GBool (*)(void*, const GoTextual*,
		GModifyValueStruct*), void* closure, GAddOrRemove = GADD);
    void	modifyValuePostcond (void (*)(void*, GoTextual*,
		GModifyValueStruct*), void* closure, GAddOrRemove = GADD);

    /* Pre/Postconds on current position change */
    void	currentPosPrecond (GBool (*)(void*, const GoTextual*, GPos),
                void* closure, GAddOrRemove = GADD);
    void	currentPosPostcond (void (*)(void*, GoTextual*, GPos),
		void* closure, GAddOrRemove = GADD);

    /* Pre/Postconds on attribute change */
    void        modifyAttributePrecond (GBool (*)(void*, const GoTextual*,
		GModifyAttributeStruct*), void* closure, GAddOrRemove = GADD);
    void        modifyAttributePostcond (void (*)(void*, GoTextual*,
		GModifyAttributeStruct*), void* closure, GAddOrRemove = GADD);

    /* Pre/Postconds on selection */
    void	selectionPrecond (GBool (*)(void*, const GoTextual*, GPos, GPos),
                void* closure, GAddOrRemove = GADD);
    void        selectionPostcond (void (*)(void*, GoTextual*, GPos, GPos),
                void* closure, GAddOrRemove = GADD);

  protected:
		GoTextualMetaclass();
    virtual	~GoTextualMetaclass();


    GoTextualCondFlags		_flags;

    GBool	_call_modify_value_precond (const GoTextual*, GModifyValueStruct*);
    void	_call_modify_value_postcond (GoTextual*, GModifyValueStruct*);
    GBool	_call_current_pos_precond (const GoTextual*, GPos);
    void	_call_current_pos_postcond (GoTextual*, GPos);
    GBool	_call_modify_attribute_precond (const GoTextual*, GModifyAttributeStruct*);
    void	_call_modify_attribute_postcond (GoTextual*, GModifyAttributeStruct*);
    GBool	_call_selection_precond (const GoTextual*, GPos, GPos);
    void	_call_selection_postcond (GoTextual*, GPos, GPos);
    virtual const GActionRec*	m_actions() const;

  private:
    virtual const char*		m_classname() const;
    virtual GoMetaclass*	m_superclass() const;
};

# endif /* _mGoTextual_h */
