/* $Id: mTreeNode.h,v 1.1.1.1 1992/03/18 10:09:33 uep Exp $ */
/*****************************************************************************
MODULE:  BULL Graphic Objects
FILE:    mTreeNod.h
AUTHOR:  Aline Jacomet
DATE:    January 11th 1991

DESCRIPTION:
         Definition of the tree node object metaclass.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/
#ifndef _mGoTreeNode_h
#define _mGoTreeNode_h

#include <Go/mNode.h>
#include <Go/TreeNode.h>


/*
 * Condition Flags
 */
struct GoTreeNodeCondFlags {
    GBool _has_attach_node_precond : 1;
    GBool _has_attach_node_postcond : 1;
    GBool _has_shift_node_precond : 1;
    GBool _has_shift_node_postcond : 1;
    GoTreeNodeCondFlags() {
	_has_attach_node_precond = GFALSE;
	_has_attach_node_postcond = GFALSE;
	_has_shift_node_precond = GFALSE;
	_has_shift_node_postcond = GFALSE;
    }
};


class GoTreeNodeMetaclass : public GoNodeMetaclass {
    friend class GoTreeNode;
    
  public:
    /*
     * Pre and Post Conditions ...;
     */
    void attachNodePrecond(GBool (*)(void*, GoTreeNode*, GoTreeNode*,
				     GoMetaclass*, GoTreeNode*, GoTree*),
			   void* closure, GAddOrRemove = GADD);
    void attachNodePostcond(void (*)(void*, GoTreeNode*, GoTreeNode*,
				     GoMetaclass*, GoTreeNode*, GoTree*),
			    void* closure, GAddOrRemove = GADD);
    void shiftNodePrecond(GBool (*)(void*, GoTreeNode*, GoTreeNode*),
			  void* closure, GAddOrRemove = GADD);
    void shiftNodePostcond(void (*)(void*, GoTreeNode*, GoTreeNode*),
			   void* closure, GAddOrRemove = GADD);
    
  protected:
    GoTreeNodeCondFlags _flags;
    
    GoTreeNodeMetaclass();
    virtual ~GoTreeNodeMetaclass();
    
    virtual void        modif_clip(Go*);

  private:
    virtual const char*      m_classname() const;
    virtual GoMetaclass*     m_superclass() const;
    virtual Go*              m_create() const;
    virtual const GActionRec* m_actions() const;
};




#endif /* _mGoTreeNode_h */
