/* $Id: machdef.h,v 1.11 1992/12/11 08:35:36 jojic Exp $ */
/*****************************************************************************
MODULE:  Machine Dependent
FILE:    machdef.h
AUTHOR:  Jacques Davy
DATE:    21 June 1990

DESCRIPTION:
    Machine Dependent definitions and includes, it is usually included
    in all files. You might be obliged to modify it.

COPYRIGHT:
  Copyright (c) 1990, 1991 Bull S.A., Paris, France
 
  Permission to use, copy, modify, distribute FREE OF CHARGE this software
  and its documentation for any purpose is hereby granted without fee, provided
  that (i) the above copyright notices and this permission notice appear in
  all copies of the software and related documentation, and (ii) the name of
  Bull may not be used in any advertising or publicity relating to the
  software without the specific, prior written permission of Bull.
  
  The permission to distribute or sublicense FOR CHARGE either this software
  (all or part) or products derived from this software is subject to prior
  licence from Bull under separate agreement.

  The information in this software is subject to change without
  notice and should not be construed as a commitment by Bull.

  Bull assumes no responsibility for the use or inability to use any of its
  software. The software is provided "AS IS" without warranty of any
  kind, and Bull expressly disclaims all implied warranties, including
  but not limited to the implied warranties of merchantability and
  fitness for a particular purpose.

  In no event shall Bull be liable for any special, incidental, indirect
  or consequential damages of any kind, or any damages whatsoever
  resulting from loss of use, data or profits, whether or not advised of
  the possibility of damage, and on any theory of liability, arising out
  of or in connection with the use or performance of this software.
******************************************************************************/

#ifndef _machdef_h
#define _machdef_h

#if defined (__MSDOS__) || defined (_MSDOS)

/* MS-DOS specifications for Go */

#include <Go/pcdef.h>

#else

/* Unix specifications for Go */
/*
 * it may be needed to define in Site.def (ExtraDefines)
 * NO_ALLOCA (if alloca does not exist on your system)
 * INLINE_NO_RETURN (if complex inline's are not implemented)
 */

#define  STATIC_OBJECT(type, variable) \
				static type variable\

#include <stddef.h>
#include <stdlib.h>
#include <memory.h>
#include <math.h>

/*
 * On mips Magnum, alloca is implemented both as a c compiler builtin and
 * as a c library function. The c++ preprocessor does not take advantage of
 * the builtin so let us use the c library version.
 */

#if defined(Mips) && ! defined(NO_ALLOCA) && ! defined (__GNUC__)
extern "C" void* alloca(int);
#endif

#ifdef SYSV
#include <float.h>
#endif

#include <values.h>
#include <string.h>

#define ArraySize(array) (sizeof(array) / sizeof(array[0]))

#ifndef PI
#define PI M_PI
#endif

typedef unsigned char 	uchar;
typedef unsigned short 	ushort;
typedef unsigned int 	uint;
typedef unsigned long 	ulong;


#ifndef NULL
#define NULL 		0
#endif

#define NULLREF		(*(void**)NULL)

#ifdef __GNUC__
# if (__GNUC__ == 1)
#     define GNUC_VERSION_1
# endif
# ifdef GNUC_VERSION_1
#     include <builtin.h>
# endif    
#endif

#ifdef GNUC_VERSION_1
#define INLINE static inline
#else
#define INLINE inline
#endif


#ifndef GNUC_VERSION_1
inline float max (float a, float b)
{
    return (a > b) ? a : b;
}
#endif

inline float max (float a, float b, float c)
{
    return (a > b) ? max(a, c) : max(b, c);
}

inline float max (float a, float b, float c, float d)
{
    return (a > b) ? max(a, c, d) : max(b, c, d);
}

#ifndef GNUC_VERSION_1
inline float min (float a, float b)
{
    return (a < b) ? a : b;
}
#endif

inline float min (float a, float b, float c)
{
    return (a < b) ? min(a, c) : min(b, c);
}

inline float min (float a, float b, float c, float d)
{
    return (a < b) ? min(a, c, d) : min(b, c, d);
}


#ifndef GNUC_VERSION_1
inline long max (long a, long b)
{
    return (a > b) ? a : b;
}
#endif

inline long max (long a, long b, long c)
{
    return (a > b) ? max(a, c) : max(b, c);
}

inline long max (long a, long b, long c, long d)
{
    return (a > b) ? max(a, c, d) : max(b, c, d);
}

#ifndef GNUC_VERSION_1
inline long min (long a, long b)
{
    return (a < b) ? a : b;
}
#endif

inline long min (long a, long b, long c)
{
    return (a < b) ? min(a, c) : min(b, c);
}

inline long min (long a, long b, long c, long d)
{
    return (a < b) ? min(a, c, d) : min(b, c, d);
}

#define bcopy __bcopy
#define bzero __bzero
#define bcmp  __bcmp

inline void __bzero(void* s, int l) { memset (s, 0, l); }
inline int  __bcmp (const void* s1, const void* s2, int l) {
    return memcmp(s1,s2,l);
}

inline void __bcopy (const void* s, void* d, int l)
{
    const char* src = (char*)s;
    char* dst = (char*)d;

    if (src < dst && src + l > dst)
    {
#define __FOUR (sizeof(long)/sizeof(char))
	if ( l & (__FOUR-1) || ((ulong)s) & 0x01 || ((ulong)d) & 0x01) {
		src += l;
		dst += l;
		while (l--)
	    		*--dst = *--src;
	} else {
		ulong* Src = (ulong*)(src + l);
		ulong* Dst = (ulong*)(dst + l);
		l = ((uint)l) / __FOUR;
		while (l--)
			*--Dst = *--Src;
	}
#undef __FOUR
    }
    else
	memcpy(d, s, l);
}


#endif /* __MSDOS__ */
#endif /* _machdef_h */
